@@ -36,7 +36,7 @@
 #include <systemd/sd-event.h>
 #include "ratbagd.h"
 #include "shared-macro.h"
-#include "shared-rbtree.h"
+#include <rbtree/shared-rbtree.h>
 
 #include "libratbag-util.h"
 
@@ -137,6 +137,28 @@ static int ratbagd_device_get_device_name(sd_bus *bus,
 	return 0;
 }
 
+static int ratbagd_device_get_device_type(sd_bus *bus,
+					  const char *path,
+					  const char *interface,
+					  const char *property,
+					  sd_bus_message *reply,
+					  void *userdata,
+					  sd_bus_error *error)
+{
+	struct ratbagd_device *device = userdata;
+	enum ratbag_device_type devicetype;
+
+	devicetype = ratbag_device_get_device_type(device->lib_device);
+	if (!devicetype) {
+		log_error("%s: device type unspecified\n",
+			  ratbagd_device_get_sysname(device));
+	}
+
+	CHECK_CALL(sd_bus_message_append(reply, "u", devicetype));
+
+	return 0;
+}
+
 static int ratbagd_device_get_profiles(sd_bus *bus,
 				       const char *path,
 				       const char *interface,
@@ -176,6 +198,11 @@ static void ratbagd_device_commit_pending(void *data)
 		log_error("error committing device (%d)\n", r);
 	if (r < 0)
 		ratbagd_device_resync(device, device->ctx->bus);
+
+	ratbagd_for_each_profile_signal(device->ctx->bus,
+					device,
+					ratbagd_profile_notify_dirty);
+
 	ratbagd_device_unref(device);
 }
 
@@ -195,13 +222,13 @@ static int ratbagd_device_commit(sd_bus_message *m,
 }
 
 static int
-ratbag_device_get_model(sd_bus *bus,
-			   const char *path,
-			   const char *interface,
-			   const char *property,
-			   sd_bus_message *reply,
-			   void *userdata,
-			   sd_bus_error *error)
+ratbagd_device_get_model(sd_bus *bus,
+			 const char *path,
+			 const char *interface,
+			 const char *property,
+			 sd_bus_message *reply,
+			 void *userdata,
+			 sd_bus_error *error)
 {
 	struct ratbagd_device *device = userdata;
 	struct ratbag_device *lib_device = device->lib_device;
@@ -220,10 +247,28 @@ ratbag_device_get_model(sd_bus *bus,
 	return sd_bus_message_append(reply, "s", model);
 }
 
+static int
+ratbagd_device_get_firmware_version(sd_bus *bus,
+				    const char *path,
+				    const char *interface,
+				    const char *property,
+				    sd_bus_message *reply,
+				    void *userdata,
+				    sd_bus_error *error)
+{
+	struct ratbagd_device *device = userdata;
+	struct ratbag_device *lib_device = device->lib_device;
+	const char* version = ratbag_device_get_firmware_version(lib_device);
+
+	return sd_bus_message_append(reply, "s", version);
+}
+
 const sd_bus_vtable ratbagd_device_vtable[] = {
 	SD_BUS_VTABLE_START(0),
-	SD_BUS_PROPERTY("Model", "s", ratbag_device_get_model, 0, SD_BUS_VTABLE_PROPERTY_CONST),
+	SD_BUS_PROPERTY("Model", "s", ratbagd_device_get_model, 0, SD_BUS_VTABLE_PROPERTY_CONST),
+	SD_BUS_PROPERTY("DeviceType", "u", ratbagd_device_get_device_type, 0, SD_BUS_VTABLE_PROPERTY_CONST),
 	SD_BUS_PROPERTY("Name", "s", ratbagd_device_get_device_name, 0, SD_BUS_VTABLE_PROPERTY_CONST),
+	SD_BUS_PROPERTY("FirmwareVersion", "s", ratbagd_device_get_firmware_version, 0, SD_BUS_VTABLE_PROPERTY_CONST),
 	SD_BUS_PROPERTY("Profiles", "ao", ratbagd_device_get_profiles, 0, SD_BUS_VTABLE_PROPERTY_CONST),
 	SD_BUS_METHOD("Commit", "", "u", ratbagd_device_commit, SD_BUS_VTABLE_UNPRIVILEGED),
 	SD_BUS_SIGNAL("Resync", "", 0),
