@@ -198,6 +198,23 @@ static int ratbagd_profile_is_active(sd_bus *bus,
 	return 0;
 }
 
+static int ratbagd_profile_is_dirty(sd_bus *bus,
+				    const char *path,
+				    const char *interface,
+				    const char *property,
+				    sd_bus_message *reply,
+				    void *userdata,
+				    sd_bus_error *error)
+{
+	const struct ratbagd_profile *profile = userdata;
+
+	const int is_dirty = ratbag_profile_is_dirty(profile->lib_profile);
+
+	CHECK_CALL(sd_bus_message_append(reply, "b", is_dirty));
+
+	return 0;
+}
+
 static int ratbagd_profile_find_button(sd_bus *bus,
 				       const char *path,
 				       const char *interface,
@@ -278,6 +295,7 @@ static int ratbagd_profile_set_active(sd_bus_message *m,
 				      void *userdata,
 				      sd_bus_error *error)
 {
+	sd_bus *bus = sd_bus_message_get_bus(m);
 	struct ratbagd_profile *profile = userdata;
 	int r;
 
@@ -285,62 +303,64 @@ static int ratbagd_profile_set_active(sd_bus_message *m,
 
 	r = ratbag_profile_set_active(profile->lib_profile);
 	if (r < 0) {
-		sd_bus *bus = sd_bus_message_get_bus(m);
 		r = ratbagd_device_resync(profile->device, bus);
 		if (r < 0)
 			return r;
 	}
 
-	ratbagd_for_each_profile_signal(sd_bus_message_get_bus(m),
+	ratbagd_for_each_profile_signal(bus,
 					profile->device,
 					ratbagd_profile_active_signal_cb);
 
+	ratbagd_profile_notify_dirty(bus, profile);
+
 	CHECK_CALL(sd_bus_reply_method_return(m, "u", 0));
 
 	return 0;
 }
 
 static int
-ratbagd_profile_set_enabled(sd_bus *bus,
-			    const char *path,
-			    const char *interface,
-			    const char *property,
-			    sd_bus_message *m,
-			    void *userdata,
-			    sd_bus_error *error)
+ratbagd_profile_set_disabled(sd_bus *bus,
+			     const char *path,
+			     const char *interface,
+			     const char *property,
+			     sd_bus_message *m,
+			     void *userdata,
+			     sd_bus_error *error)
 {
 	struct ratbagd_profile *profile = userdata;
-	int enabled;
 	int r;
+	int disabled;
 
-	CHECK_CALL(sd_bus_message_read(m, "b", &enabled));
+	CHECK_CALL(sd_bus_message_read(m, "b", &disabled));
 
-	r = ratbag_profile_set_enabled(profile->lib_profile, enabled);
+	r = ratbag_profile_set_enabled(profile->lib_profile, !disabled);
 	if (r == 0) {
-		sd_bus *bus = sd_bus_message_get_bus(m);
 		sd_bus_emit_properties_changed(bus,
 					       profile->path,
 					       RATBAGD_NAME_ROOT ".Profile",
-					       "Enabled",
+					       "Disabled",
 					       NULL);
+
+		ratbagd_profile_notify_dirty(bus, profile);
 	}
 
 	return 0;
 }
 
 static int
-ratbagd_profile_is_enabled(sd_bus *bus,
-			   const char *path,
-			   const char *interface,
-			   const char *property,
-			   sd_bus_message *reply,
-			   void *userdata,
-			   sd_bus_error *error)
+ratbagd_profile_is_disabled(sd_bus *bus,
+			    const char *path,
+			    const char *interface,
+			    const char *property,
+			    sd_bus_message *reply,
+			    void *userdata,
+			    sd_bus_error *error)
 {
 	struct ratbagd_profile *profile = userdata;
-	int enabled = ratbag_profile_is_enabled(profile->lib_profile) != 0;
+	int disabled = !ratbag_profile_is_enabled(profile->lib_profile);
 
-	CHECK_CALL(sd_bus_message_append(reply, "b", enabled));
+	CHECK_CALL(sd_bus_message_append(reply, "b", disabled));
 
 	return 0;
 }
@@ -368,6 +388,8 @@ ratbagd_profile_set_name(sd_bus *bus,
 					       RATBAGD_NAME_ROOT ".Profile",
 					       "Name",
 					       NULL);
+
+		ratbagd_profile_notify_dirty(bus, profile);
 	}
 
 	return 0;
@@ -455,6 +477,40 @@ ratbagd_profile_get_report_rate(sd_bus *bus,
 	return sd_bus_message_append(reply, "u", rate);
 }
 
+static int
+ratbagd_profile_get_angle_snapping(sd_bus *bus,
+				   const char *path,
+				   const char *interface,
+				   const char *property,
+				   sd_bus_message *reply,
+				   void *userdata,
+				   sd_bus_error *error)
+{
+	struct ratbagd_profile *profile = userdata;
+	struct ratbag_profile *lib_profile = profile->lib_profile;
+	int value;
+
+	value = ratbag_profile_get_angle_snapping(lib_profile);
+	return sd_bus_message_append(reply, "i", value);
+}
+
+static int
+ratbagd_profile_get_debounce(sd_bus *bus,
+			     const char *path,
+			     const char *interface,
+			     const char *property,
+			     sd_bus_message *reply,
+			     void *userdata,
+			     sd_bus_error *error)
+{
+	struct ratbagd_profile *profile = userdata;
+	struct ratbag_profile *lib_profile = profile->lib_profile;
+	int value;
+
+	value = ratbag_profile_get_debounce(lib_profile);
+	return sd_bus_message_append(reply, "i", value);
+}
+
 static int
 ratbagd_profile_get_report_rates(sd_bus *bus,
 				 const char *path,
@@ -488,6 +544,37 @@ ratbagd_profile_get_report_rates(sd_bus *bus,
 	return sd_bus_message_close_container(reply);
 }
 
+static int
+ratbagd_profile_get_debounces(sd_bus *bus,
+			      const char *path,
+			      const char *interface,
+			      const char *property,
+			      sd_bus_message *reply,
+			      void *userdata,
+			      sd_bus_error *error)
+{
+	struct ratbagd_profile *profile = userdata;
+	struct ratbag_profile *lib_profile = profile->lib_profile;
+	unsigned int debounces[8];
+	unsigned int ndebounces = ARRAY_LENGTH(debounces);
+	int r;
+
+	r = sd_bus_message_open_container(reply, 'a', "u");
+	if (r < 0)
+		return r;
+
+	ndebounces = ratbag_profile_get_debounce_list(lib_profile, debounces, ndebounces);
+	assert(ndebounces <= ARRAY_LENGTH(debounces));
+
+	for (unsigned int i = 0; i < ndebounces; i++) {
+		verify_unsigned_int(debounces[i]);
+		r = sd_bus_message_append(reply, "u", debounces[i]);
+		if (r < 0)
+			return r;
+	}
+
+	return sd_bus_message_close_container(reply);
+}
 
 static int
 ratbagd_profile_set_report_rate(sd_bus *bus,
@@ -507,17 +594,85 @@ ratbagd_profile_set_report_rate(sd_bus *bus,
 		return r;
 
 	/* basic sanity check */
-	if (rate > 8000)
+	if (rate < 125) {
+		rate = 125;
+	} else if (rate > 8000) {
 		rate = 8000;
+	}
 
 	r = ratbag_profile_set_report_rate(profile->lib_profile, rate);
 	if (r == 0) {
-		sd_bus *bus = sd_bus_message_get_bus(m);
 		sd_bus_emit_properties_changed(bus,
 					       profile->path,
 					       RATBAGD_NAME_ROOT ".Profile",
 					       "ReportRate",
 					       NULL);
+
+		ratbagd_profile_notify_dirty(bus, profile);
+	}
+
+	return 0;
+}
+
+static int
+ratbagd_profile_set_angle_snapping(sd_bus *bus,
+				   const char *path,
+				   const char *interface,
+				   const char *property,
+				   sd_bus_message *m,
+				   void *userdata,
+				   sd_bus_error *error)
+{
+	struct ratbagd_profile *profile = userdata;
+	int value;
+	int r;
+
+	r = sd_bus_message_read(m, "i", &value);
+	if (r < 0)
+		return r;
+
+	r = ratbag_profile_set_angle_snapping(profile->lib_profile, value);
+	if (r == 0) {
+		sd_bus *bus = sd_bus_message_get_bus(m);
+		sd_bus_emit_properties_changed(bus,
+					       profile->path,
+					       RATBAGD_NAME_ROOT ".Profile",
+					       "AngleSnapping",
+					       NULL);
+
+		ratbagd_profile_notify_dirty(bus, profile);
+	}
+
+	return 0;
+}
+
+static int
+ratbagd_profile_set_debounce(sd_bus *bus,
+			     const char *path,
+			     const char *interface,
+			     const char *property,
+			     sd_bus_message *m,
+			     void *userdata,
+			     sd_bus_error *error)
+{
+	struct ratbagd_profile *profile = userdata;
+	int value;
+	int r;
+
+	r = sd_bus_message_read(m, "i", &value);
+	if (r < 0)
+		return r;
+
+	r = ratbag_profile_set_debounce(profile->lib_profile, value);
+	if (r == 0) {
+		sd_bus *bus = sd_bus_message_get_bus(m);
+		sd_bus_emit_properties_changed(bus,
+					       profile->path,
+					       RATBAGD_NAME_ROOT ".Profile",
+					       "Debounce",
+					       NULL);
+
+		ratbagd_profile_notify_dirty(bus, profile);
 	}
 
 	return 0;
@@ -529,9 +684,9 @@ const sd_bus_vtable ratbagd_profile_vtable[] = {
 				 ratbagd_profile_get_name,
 				 ratbagd_profile_set_name, 0,
 				 SD_BUS_VTABLE_UNPRIVILEGED|SD_BUS_VTABLE_PROPERTY_EMITS_CHANGE),
-	SD_BUS_WRITABLE_PROPERTY("Enabled", "b",
-				 ratbagd_profile_is_enabled,
-				 ratbagd_profile_set_enabled, 0,
+	SD_BUS_WRITABLE_PROPERTY("Disabled", "b",
+				 ratbagd_profile_is_disabled,
+				 ratbagd_profile_set_disabled, 0,
 				 SD_BUS_VTABLE_UNPRIVILEGED|SD_BUS_VTABLE_PROPERTY_EMITS_CHANGE),
 	SD_BUS_PROPERTY("Index", "u", NULL, offsetof(struct ratbagd_profile, index), SD_BUS_VTABLE_PROPERTY_CONST),
 	SD_BUS_PROPERTY("Capabilities", "au", ratbagd_profile_get_capabilities, 0, SD_BUS_VTABLE_PROPERTY_CONST),
@@ -539,11 +694,21 @@ const sd_bus_vtable ratbagd_profile_vtable[] = {
 	SD_BUS_PROPERTY("Buttons", "ao", ratbagd_profile_get_buttons, 0, 0),
 	SD_BUS_PROPERTY("Leds", "ao", ratbagd_profile_get_leds, 0, 0),
 	SD_BUS_PROPERTY("IsActive", "b", ratbagd_profile_is_active, 0, SD_BUS_VTABLE_PROPERTY_EMITS_CHANGE),
+	SD_BUS_PROPERTY("IsDirty", "b", ratbagd_profile_is_dirty, 0, SD_BUS_VTABLE_PROPERTY_EMITS_CHANGE),
 	SD_BUS_WRITABLE_PROPERTY("ReportRate", "u",
 				 ratbagd_profile_get_report_rate,
 				 ratbagd_profile_set_report_rate, 0,
 				 SD_BUS_VTABLE_UNPRIVILEGED|SD_BUS_VTABLE_PROPERTY_EMITS_CHANGE),
+	SD_BUS_WRITABLE_PROPERTY("AngleSnapping", "i",
+				 ratbagd_profile_get_angle_snapping,
+				 ratbagd_profile_set_angle_snapping, 0,
+				 SD_BUS_VTABLE_UNPRIVILEGED|SD_BUS_VTABLE_PROPERTY_EMITS_CHANGE),
+	SD_BUS_WRITABLE_PROPERTY("Debounce", "i",
+				 ratbagd_profile_get_debounce,
+				 ratbagd_profile_set_debounce, 0,
+				 SD_BUS_VTABLE_UNPRIVILEGED|SD_BUS_VTABLE_PROPERTY_EMITS_CHANGE),
 	SD_BUS_PROPERTY("ReportRates", "au", ratbagd_profile_get_report_rates, 0, SD_BUS_VTABLE_PROPERTY_CONST),
+	SD_BUS_PROPERTY("Debounces", "au", ratbagd_profile_get_debounces, 0, SD_BUS_VTABLE_PROPERTY_CONST),
 	SD_BUS_METHOD("SetActive", "", "u", ratbagd_profile_set_active, SD_BUS_VTABLE_UNPRIVILEGED),
 	SD_BUS_VTABLE_END,
 };
@@ -941,3 +1106,15 @@ int ratbagd_profile_resync(sd_bus *bus,
 					      "IsActive",
 					      NULL);
 }
+
+int ratbagd_profile_notify_dirty(sd_bus *bus,
+				 struct ratbagd_profile *profile)
+{
+	(void)sd_bus_emit_properties_changed(bus,
+					     profile->path,
+					     RATBAGD_NAME_ROOT ".Profile",
+					     "IsDirty",
+					     NULL);
+
+	return 0;
+}
