@@ -280,19 +280,6 @@ struct gskill_macro_report {
 _Static_assert(sizeof(struct gskill_macro_report) == GSKILL_REPORT_SIZE_MACRO,
 	       "Size of gskill_macro_report is wrong");
 
-enum ratbag_button_type gskill_button_type_mapping[] = {
-	RATBAG_BUTTON_TYPE_LEFT,
-	RATBAG_BUTTON_TYPE_RIGHT,
-	RATBAG_BUTTON_TYPE_MIDDLE,
-	RATBAG_BUTTON_TYPE_THUMB,
-	RATBAG_BUTTON_TYPE_THUMB2,
-	RATBAG_BUTTON_TYPE_RESOLUTION_CYCLE_UP,
-	RATBAG_BUTTON_TYPE_THUMB3,
-	RATBAG_BUTTON_TYPE_THUMB4,
-	RATBAG_BUTTON_TYPE_WHEEL_UP,
-	RATBAG_BUTTON_TYPE_WHEEL_DOWN,
-};
-
 struct gskill_button_function_mapping {
 	enum gskill_button_function_type type;
 	struct ratbag_button_action action;
@@ -667,7 +654,7 @@ gskill_macro_code_from_event(struct ratbag_device *device,
 	uint8_t macro_code;
 
 	/*
-	 * The miscellanious keycodes are ORd with 0x70 to indicate press, 0xF0
+	 * The miscellaneous keycodes are ORd with 0x70 to indicate press, 0xF0
 	 * to indicate release
 	 */
 	if (event->type == RATBAG_MACRO_EVENT_KEY_PRESSED)
@@ -987,7 +974,7 @@ gskill_read_resolutions(struct ratbag_profile *profile,
 
 	hz = GSKILL_MAX_POLLING_RATE / (report->polling_rate + 1);
 	ratbag_profile_set_report_rate_list(profile, rates, ARRAY_LENGTH(rates));
-	ratbag_profile_set_report_rate(profile, hz);
+	profile->hz = hz;
 
 	for (i = 0; i < report->dpi_num; i++) {
 		_cleanup_resolution_ struct ratbag_resolution *resolution = NULL;
@@ -1162,8 +1149,7 @@ gskill_read_button(struct ratbag_button *button)
 	struct gskill_button_cfg *bcfg = &report->btn_cfgs[button->index];
 	struct ratbag_button_action *act = &button->action;
 
-	button->type = gskill_button_type_mapping[button->index];
-
+	ratbag_button_enable_action_type(button, RATBAG_BUTTON_ACTION_TYPE_NONE);
 	ratbag_button_enable_action_type(button, RATBAG_BUTTON_ACTION_TYPE_BUTTON);
 	ratbag_button_enable_action_type(button, RATBAG_BUTTON_ACTION_TYPE_SPECIAL);
 	ratbag_button_enable_action_type(button, RATBAG_BUTTON_ACTION_TYPE_KEY);
@@ -1193,33 +1179,34 @@ gskill_read_button(struct ratbag_button *button)
 	case GSKILL_BUTTON_FUNCTION_MOUSE:
 		act->type = RATBAG_BUTTON_ACTION_TYPE_BUTTON;
 
+		/* FIXME: There is almost no chance this is correct. */
 		switch (bcfg->params.mouse.button_mask) {
 		case GSKILL_BTN_MASK_LEFT:
-			act->action.button = RATBAG_BUTTON_TYPE_LEFT;
+			act->action.button = 1;
 			break;
 		case GSKILL_BTN_MASK_RIGHT:
-			act->action.button = RATBAG_BUTTON_TYPE_RIGHT;
+			act->action.button = 3;
 			break;
 		case GSKILL_BTN_MASK_MIDDLE:
-			act->action.button = RATBAG_BUTTON_TYPE_MIDDLE;
+			act->action.button = 2;
 			break;
 		case GSKILL_BTN_MASK_SIDE:
-			act->action.button = RATBAG_BUTTON_TYPE_SIDE;
+			act->action.button = 15;
 			break;
 		case GSKILL_BTN_MASK_EXTRA:
-			act->action.button = RATBAG_BUTTON_TYPE_EXTRA;
+			act->action.button = 14;
 			break;
 		}
 		break;
 	case GSKILL_BUTTON_FUNCTION_KBD:
 		act->type = RATBAG_BUTTON_ACTION_TYPE_KEY;
-		act->action.key.key =
+		act->action.key =
 			ratbag_hidraw_get_keycode_from_keyboard_usage(
 			    device, bcfg->params.kbd.hid_code);
 		break;
 	case GSKILL_BUTTON_FUNCTION_CONSUMER:
 		act->type = RATBAG_BUTTON_ACTION_TYPE_KEY;
-		act->action.key.key =
+		act->action.key =
 			ratbag_hidraw_get_keycode_from_consumer_usage(
 			    device, bcfg->params.consumer.code);
 		break;
@@ -1297,35 +1284,37 @@ gskill_update_button(struct ratbag_button *button)
 	case RATBAG_BUTTON_ACTION_TYPE_BUTTON:
 		bcfg->type = GSKILL_BUTTON_FUNCTION_MOUSE;
 
+		/* FIXME: There is almost no chance this is correct. */
 		switch (action->action.button) {
-		case RATBAG_BUTTON_TYPE_LEFT:
+		case 1:
 			bcfg->params.mouse.button_mask = GSKILL_BTN_MASK_LEFT;
 			break;
-		case RATBAG_BUTTON_TYPE_RIGHT:
+		case 3:
 			bcfg->params.mouse.button_mask = GSKILL_BTN_MASK_RIGHT;
 			break;
-		case RATBAG_BUTTON_TYPE_MIDDLE:
+		case 2:
 			bcfg->params.mouse.button_mask = GSKILL_BTN_MASK_MIDDLE;
 			break;
-		case RATBAG_BUTTON_TYPE_SIDE:
+		case 15:
 			bcfg->params.mouse.button_mask = GSKILL_BTN_MASK_SIDE;
 			break;
-		case RATBAG_BUTTON_TYPE_EXTRA:
+		case 14:
 			bcfg->params.mouse.button_mask = GSKILL_BTN_MASK_EXTRA;
 			break;
 		default:
 			return -EINVAL;
 		}
+
 		break;
 	case RATBAG_BUTTON_ACTION_TYPE_KEY:
 		code = ratbag_hidraw_get_keyboard_usage_from_keycode(
-		    device, action->action.key.key);
+		    device, action->action.key);
 		if (code) {
 			bcfg->type = GSKILL_BUTTON_FUNCTION_KBD;
 			bcfg->params.kbd.hid_code = code;
 		} else {
 			code = ratbag_hidraw_get_consumer_usage_from_keycode(
-			    device, action->action.key.key);
+			    device, action->action.key);
 
 			bcfg->type = GSKILL_BUTTON_FUNCTION_CONSUMER;
 			bcfg->params.consumer.code = code;
@@ -1451,7 +1440,7 @@ gskill_commit(struct ratbag_device *device)
 	 * G.Skill mice only have a concept of how many profiles are enabled,
 	 * not which ones are and aren't enabled. So in order to provide the
 	 * ability to disable individual profiles we need to only write the
-	 * enabled profiles and make sure no holes are left inbetween profiles
+	 * enabled profiles and make sure no holes are left in between profiles
 	 */
 	for (i = 0; i < GSKILL_PROFILE_MAX; i++) {
 		profile = ratbag_device_get_profile(device, i);
