@@ -177,13 +177,13 @@ hidpp10drv_map_button(struct ratbag_device *device,
 		break;
 	case PROFILE_BUTTON_TYPE_KEYS:
 		button->action.type = RATBAG_BUTTON_ACTION_TYPE_KEY;
-		button->action.action.key.key = ratbag_hidraw_get_keycode_from_keyboard_usage(device,
+		button->action.action.key = ratbag_hidraw_get_keycode_from_keyboard_usage(device,
 							profile.buttons[button->index].keys.key);
 		modifiers = profile.buttons[button->index].keys.modifier_flags;
 		break;
 	case PROFILE_BUTTON_TYPE_CONSUMER_CONTROL:
 		button->action.type = RATBAG_BUTTON_ACTION_TYPE_KEY;
-		button->action.action.key.key = ratbag_hidraw_get_keycode_from_consumer_usage(device,
+		button->action.action.key = ratbag_hidraw_get_keycode_from_consumer_usage(device,
 							profile.buttons[button->index].consumer_control.consumer_control);
 		break;
 	case PROFILE_BUTTON_TYPE_SPECIAL:
@@ -202,7 +202,7 @@ hidpp10drv_map_button(struct ratbag_device *device,
 	}
 
 	if (button->action.type == RATBAG_BUTTON_ACTION_TYPE_KEY) {
-		ret = ratbag_button_macro_new_from_keycode(button, button->action.action.key.key, modifiers);
+		ret = ratbag_button_macro_new_from_keycode(button, button->action.action.key, modifiers);
 		if (ret < 0) {
 			log_error(device->ratbag, "hidpp10: error while reading button %d\n", button->index);
 			button->action.type = RATBAG_BUTTON_ACTION_TYPE_NONE;
@@ -215,86 +215,21 @@ hidpp10drv_map_button(struct ratbag_device *device,
 static void
 hidpp10drv_read_button(struct ratbag_button *button)
 {
-	enum ratbag_button_type type = RATBAG_BUTTON_TYPE_UNKNOWN;
 	struct ratbag_device *device = button->profile->device;
 	struct hidpp10drv_data *drv_data = ratbag_get_drv_data(device);
 	struct hidpp10_device *hidpp10 = drv_data->dev;
 
 	switch (hidpp10->profile_type) {
 	case HIDPP10_PROFILE_G500:
-		switch (button->index) {
-		case 0: type = RATBAG_BUTTON_TYPE_LEFT; break;
-		case 1: type = RATBAG_BUTTON_TYPE_MIDDLE; break;
-		case 2: type = RATBAG_BUTTON_TYPE_RIGHT; break;
-		case 3: type = RATBAG_BUTTON_TYPE_THUMB; break;
-		case 4: type = RATBAG_BUTTON_TYPE_THUMB2; break;
-		case 5: type = RATBAG_BUTTON_TYPE_THUMB3; break;
-		case 6: type = RATBAG_BUTTON_TYPE_WHEEL_LEFT; break;
-		case 7: type = RATBAG_BUTTON_TYPE_WHEEL_RIGHT; break;
-		case 8: type = RATBAG_BUTTON_TYPE_RESOLUTION_UP; break;
-		case 9: type = RATBAG_BUTTON_TYPE_RESOLUTION_DOWN; break;
-		case 10:
-		case 11:
-		case 12: /* these don't actually exist on the device */
-			type = RATBAG_BUTTON_TYPE_UNKNOWN; break;
-		default:
-			break;
-		}
-		hidpp10drv_map_button(device, hidpp10, button);
-		break;
 	case HIDPP10_PROFILE_G700:
-		switch (button->index) {
-		case 0: type = RATBAG_BUTTON_TYPE_LEFT; break;
-		case 1: type = RATBAG_BUTTON_TYPE_MIDDLE; break;
-		case 2: type = RATBAG_BUTTON_TYPE_RIGHT; break;
-		case 3: type = RATBAG_BUTTON_TYPE_THUMB; break;
-		case 4: type = RATBAG_BUTTON_TYPE_THUMB2; break;
-		case 5: type = RATBAG_BUTTON_TYPE_THUMB3; break;
-		case 6: type = RATBAG_BUTTON_TYPE_THUMB4; break;
-		case 7: type = RATBAG_BUTTON_TYPE_RESOLUTION_CYCLE_UP; break;
-		case 8: type = RATBAG_BUTTON_TYPE_RESOLUTION_DOWN; break;
-		case 9: type = RATBAG_BUTTON_TYPE_RESOLUTION_UP; break;
-		case 10: type = RATBAG_BUTTON_TYPE_PROFILE_CYCLE_UP; break;
-		case 11: type = RATBAG_BUTTON_TYPE_WHEEL_LEFT; break;
-		case 12: type = RATBAG_BUTTON_TYPE_WHEEL_RIGHT; break;
-		default:
-			break;
-		}
-		hidpp10drv_map_button(device, hidpp10, button);
-		break;
 	case HIDPP10_PROFILE_G9:
-		switch (button->index) {
-		case 0: type = RATBAG_BUTTON_TYPE_LEFT; break;
-		case 1: type = RATBAG_BUTTON_TYPE_RIGHT; break;
-		case 2: type = RATBAG_BUTTON_TYPE_MIDDLE; break;
-		case 3: type = RATBAG_BUTTON_TYPE_THUMB; break;
-		case 4: type = RATBAG_BUTTON_TYPE_THUMB2; break;
-		case 5: type = RATBAG_BUTTON_TYPE_UNKNOWN; break;
-		case 6: type = RATBAG_BUTTON_TYPE_WHEEL_LEFT; break;
-		case 7: type = RATBAG_BUTTON_TYPE_WHEEL_RIGHT; break;
-		case 8: type = RATBAG_BUTTON_TYPE_RESOLUTION_UP; break;
-		case 9: type = RATBAG_BUTTON_TYPE_RESOLUTION_DOWN; break;
-		case 10:
-		case 11:
-		case 12: /* these don't actually exist on the device */
-			type = RATBAG_BUTTON_TYPE_UNKNOWN; break;
-		default:
-			break;
-		}
 		hidpp10drv_map_button(device, hidpp10, button);
 		break;
 	default:
-		switch (button->index) {
-		case 0: type = RATBAG_BUTTON_TYPE_LEFT; break;
-		case 1: type = RATBAG_BUTTON_TYPE_MIDDLE; break;
-		case 2: type = RATBAG_BUTTON_TYPE_RIGHT; break;
-		default:
-			break;
-		}
+		break;
 	}
 
-	button->type = type;
-
+	ratbag_button_enable_action_type(button, RATBAG_BUTTON_ACTION_TYPE_NONE);
 	ratbag_button_enable_action_type(button, RATBAG_BUTTON_ACTION_TYPE_BUTTON);
 	ratbag_button_enable_action_type(button, RATBAG_BUTTON_ACTION_TYPE_KEY);
 	ratbag_button_enable_action_type(button, RATBAG_BUTTON_ACTION_TYPE_SPECIAL);
@@ -321,9 +256,9 @@ hidpp10drv_write_button(struct hidpp10_profile *profile,
 		profile->buttons[button->index].button.button = action->action.button;
 		break;
 	case RATBAG_BUTTON_ACTION_TYPE_KEY:
-		code = ratbag_hidraw_get_keyboard_usage_from_keycode(device, action->action.key.key);
+		code = ratbag_hidraw_get_keyboard_usage_from_keycode(device, action->action.key);
 		if (code == 0) {
-			code = ratbag_hidraw_get_consumer_usage_from_keycode(device, action->action.key.key);
+			code = ratbag_hidraw_get_consumer_usage_from_keycode(device, action->action.key);
 			if (code == 0)
 				return -EINVAL;
 
@@ -495,7 +430,7 @@ hidpp10drv_read_profile(struct ratbag_profile *profile)
 	}
 
 	ratbag_profile_set_report_rate_list(profile, rates, ARRAY_LENGTH(rates));
-	ratbag_profile_set_report_rate(profile, p.refresh_rate);
+	profile->hz = p.refresh_rate;
 
 	ratbag_profile_for_each_button(profile, button)
 		hidpp10drv_read_button(button);
@@ -636,7 +571,6 @@ hidpp10drv_probe(struct ratbag_device *device)
 	const char *typestr;
 	int device_idx = HIDPP_WIRED_DEVICE_IDX;
 	unsigned int profile_count = 1;
-	struct ratbag_profile *profile;
 
 	rc = ratbag_find_hidraw(device, hidpp10drv_test_hidraw);
 	if (rc)
@@ -655,7 +589,15 @@ hidpp10drv_probe(struct ratbag_device *device)
 		else if (strcasecmp("G9", typestr) == 0)
 			type = HIDPP10_PROFILE_G9;
 
-		profile_count = ratbag_device_data_hidpp10_get_profile_count(device->data);
+		rc = ratbag_device_data_hidpp10_get_profile_count(device->data);
+		if (rc == -1) {
+			log_error(device->ratbag,
+				  "Device %s has no profile count set, even though profiles are enabled. "
+				  "Please adjust the .device file.\n",
+				  device->name);
+		} else {
+			profile_count = (unsigned int)rc;
+		};
 	}
 
 	device_idx = ratbag_device_data_hidpp10_get_index(device->data);
@@ -724,8 +666,11 @@ hidpp10drv_probe(struct ratbag_device *device)
 		profile->is_active = true;
 	}
 
-	ratbag_device_for_each_profile(device, profile)
-		hidpp10drv_read_profile(profile);
+	{
+		struct ratbag_profile *profile;
+		ratbag_device_for_each_profile(device, profile)
+			hidpp10drv_read_profile(profile);
+	}
 
 	if (device->num_profiles == 1) {
 		_cleanup_profile_ struct ratbag_profile *profile;
