@@ -55,7 +55,7 @@
 #define HIDPP_CAP_COLOR_LED_EFFECTS_8070		(1 << 5)
 #define HIDPP_CAP_ONBOARD_PROFILES_8100			(1 << 6)
 #define HIDPP_CAP_LED_SW_CONTROL_1300			(1 << 7)
-#define HIDPP_CAP_ADJUSTIBLE_REPORT_RATE_8060		(1 << 8)
+#define HIDPP_CAP_ADJUSTABLE_REPORT_RATE_8060		(1 << 8)
 #define HIDPP_CAP_BATTERY_VOLTAGE_1001			(1 << 9)
 #define HIDPP_CAP_RGB_EFFECTS_8071			(1 << 10)
 
@@ -104,11 +104,11 @@ hidpp20drv_read_button_1b04(struct ratbag_button *button)
 		  mapping,
 		  control->reporting.divert || control->reporting.persist ? "(redirected) " : "",
 		  __FILE__, __LINE__);
-	button->type = hidpp20_1b04_get_physical_mapping(control->task_id);
 	action = hidpp20_1b04_get_logical_mapping(mapping);
 	if (action)
 		ratbag_button_set_action(button, action);
 
+	ratbag_button_enable_action_type(button, RATBAG_BUTTON_ACTION_TYPE_NONE);
 	ratbag_button_enable_action_type(button, RATBAG_BUTTON_ACTION_TYPE_BUTTON);
 	ratbag_button_enable_action_type(button, RATBAG_BUTTON_ACTION_TYPE_KEY);
 	ratbag_button_enable_action_type(button, RATBAG_BUTTON_ACTION_TYPE_SPECIAL);
@@ -213,19 +213,22 @@ hidpp20drv_read_button_8100(struct ratbag_button *button)
 	switch (profile->buttons[button->index].any.type) {
 	case HIDPP20_BUTTON_HID_TYPE:
 		switch (profile->buttons[button->index].subany.subtype) {
+		case HIDPP20_BUTTON_HID_TYPE_NOOP:
+			button->action.type = RATBAG_BUTTON_ACTION_TYPE_NONE;
+			break;
 		case HIDPP20_BUTTON_HID_TYPE_MOUSE:
 			button->action.type = RATBAG_BUTTON_ACTION_TYPE_BUTTON;
 			button->action.action.button = profile->buttons[button->index].button.buttons;
 			break;
 		case HIDPP20_BUTTON_HID_TYPE_KEYBOARD:
 			button->action.type = RATBAG_BUTTON_ACTION_TYPE_KEY;
-			button->action.action.key.key = ratbag_hidraw_get_keycode_from_keyboard_usage(device,
+			button->action.action.key = ratbag_hidraw_get_keycode_from_keyboard_usage(device,
 								profile->buttons[button->index].keyboard_keys.key);
 			modifiers = profile->buttons[button->index].keyboard_keys.modifier_flags;
 			break;
 		case HIDPP20_BUTTON_HID_TYPE_CONSUMER_CONTROL:
 			button->action.type = RATBAG_BUTTON_ACTION_TYPE_KEY;
-			button->action.action.key.key = ratbag_hidraw_get_keycode_from_consumer_usage(device,
+			button->action.action.key = ratbag_hidraw_get_keycode_from_consumer_usage(device,
 								profile->buttons[button->index].consumer_control.consumer_control);
 			break;
 		}
@@ -245,10 +248,8 @@ hidpp20drv_read_button_8100(struct ratbag_button *button)
 	}
 
 	if (button->action.type == RATBAG_BUTTON_ACTION_TYPE_KEY) {
-		int rc;
-
 		rc = ratbag_button_macro_new_from_keycode(button,
-							  button->action.action.key.key,
+							  button->action.action.key,
 							  modifiers);
 		if (rc < 0) {
 			log_error(device->ratbag,
@@ -258,6 +259,7 @@ hidpp20drv_read_button_8100(struct ratbag_button *button)
 		}
 	}
 
+	ratbag_button_enable_action_type(button, RATBAG_BUTTON_ACTION_TYPE_NONE);
 	ratbag_button_enable_action_type(button, RATBAG_BUTTON_ACTION_TYPE_BUTTON);
 	ratbag_button_enable_action_type(button, RATBAG_BUTTON_ACTION_TYPE_KEY);
 	ratbag_button_enable_action_type(button, RATBAG_BUTTON_ACTION_TYPE_SPECIAL);
@@ -287,23 +289,6 @@ hidpp20drv_read_led_1300(struct ratbag_led *led, struct hidpp20drv_data* data)
 
 	led->colordepth = RATBAG_LED_COLORDEPTH_MONOCHROME;
 
-	switch (info->type)
-	{
-	case HIDPP20_LED_TYPE_LOGO:
-		led->type = RATBAG_LED_TYPE_LOGO;
-		break;
-	case HIDPP20_LED_TYPE_BATTERY:
-		led->type = RATBAG_LED_TYPE_BATTERY;
-		break;
-	case HIDPP20_LED_TYPE_DPI:
-		led->type = RATBAG_LED_TYPE_DPI;
-		break;
-	case HIDPP20_LED_TYPE_PROFILE:
-	case HIDPP20_LED_TYPE_COSMETIC:
-		led->type = RATBAG_LED_TYPE_SIDE;
-		break;
-	}
-
 	switch (state.mode)
 	{
 	case HIDPP20_LED_MODE_OFF:
@@ -369,9 +354,6 @@ hidpp20drv_read_led_8070(struct ratbag_led *led, struct hidpp20drv_data* drv_dat
 		break;
 	}
 
-	/* pre-filled, only override if unknown */
-	if (led->type == RATBAG_LED_TYPE_UNKNOWN)
-		led->type = hidpp20_led_get_location_mapping(led_info->location);
 	led->color.red = h_led->color.red;
 	led->color.green = h_led->color.green;
 	led->color.blue = h_led->color.blue;
@@ -446,9 +428,6 @@ hidpp20drv_read_led_8071(struct ratbag_led *led, struct hidpp20drv_data* drv_dat
 		break;
 	}
 
-	/* pre-filled, only override if unknown */
-	if (led->type == RATBAG_LED_TYPE_UNKNOWN)
-		led->type = hidpp20_led_get_location_mapping(cluster_info.location);
 	led->color.red = h_led->color.red;
 	led->color.green = h_led->color.green;
 	led->color.blue = h_led->color.blue;
@@ -591,10 +570,10 @@ hidpp20drv_update_button_8100(struct ratbag_button *button)
 	case RATBAG_BUTTON_ACTION_TYPE_KEY:
 		type = HIDPP20_BUTTON_HID_TYPE;
 		subtype = HIDPP20_BUTTON_HID_TYPE_KEYBOARD;
-		code = ratbag_hidraw_get_keyboard_usage_from_keycode(device, action->action.key.key);
+		code = ratbag_hidraw_get_keyboard_usage_from_keycode(device, action->action.key);
 		if (code == 0) {
 			subtype = HIDPP20_BUTTON_HID_TYPE_CONSUMER_CONTROL;
-			code = ratbag_hidraw_get_consumer_usage_from_keycode(device, action->action.key.key);
+			code = ratbag_hidraw_get_consumer_usage_from_keycode(device, action->action.key);
 			if (code == 0)
 				return -EINVAL;
 		}
@@ -612,6 +591,9 @@ hidpp20drv_update_button_8100(struct ratbag_button *button)
 		profile->buttons[button->index].special.type = HIDPP20_BUTTON_SPECIAL;
 		profile->buttons[button->index].special.special = code;
 		break;
+	case RATBAG_BUTTON_ACTION_TYPE_NONE:
+		profile->buttons[button->index].disabled.type = HIDPP20_BUTTON_HID_TYPE_NOOP;
+		break;
 	default:
 		return -ENOTSUP;
 	}
@@ -702,7 +684,7 @@ hidpp20drv_update_led_8070_8071(struct ratbag_led *led, struct ratbag_profile* p
 				struct hidpp20drv_data *drv_data)
 {
 	struct hidpp20_profile *h_profile;
-	struct hidpp20_led h_led_val;
+	struct hidpp20_led h_led_val = {0};
 	struct hidpp20_led *h_led = &h_led_val;
 
 	if (drv_data->capabilities & HIDPP_CAP_ONBOARD_PROFILES_8100) {
@@ -760,22 +742,6 @@ hidpp20drv_update_led(struct ratbag_led *led)
 	return RATBAG_ERROR_CAPABILITY;
 }
 
-static int
-hidpp20drv_current_profile(struct ratbag_device *device)
-{
-	struct hidpp20drv_data *drv_data = ratbag_get_drv_data((struct ratbag_device *)device);
-	int rc;
-
-	if (!(drv_data->capabilities & HIDPP_CAP_ONBOARD_PROFILES_8100))
-		return 0;
-
-	rc = hidpp20_onboard_profiles_get_current_profile(drv_data->dev);
-	if (rc < 0)
-		return rc;
-
-	return rc - 1;
-}
-
 static int
 hidpp20drv_set_current_profile(struct ratbag_device *device, unsigned int index)
 {
@@ -955,7 +921,7 @@ hidpp20drv_read_resolution_dpi(struct ratbag_profile *profile)
 			ratbag_resolution_set_dpi_list(res, &default_dpi, 1);
 	}
 
-	if (drv_data->capabilities & HIDPP_CAP_ADJUSTIBLE_REPORT_RATE_8060) {
+	if (drv_data->capabilities & HIDPP_CAP_ADJUSTABLE_REPORT_RATE_8060) {
 		rc = hidpp20drv_read_report_rate_8060(device);
 		if (rc < 0 && drv_data->report_rates[0] == 0)
 			return rc;
@@ -1000,6 +966,13 @@ hidpp20drv_update_resolution_dpi(struct ratbag_resolution *resolution,
 	int i;
 	int dpi = dpi_x; /* dpi_x == dpi_y if we don't have the individual resolution cap */
 
+	if (resolution->is_disabled) {
+		if (!ratbag_resolution_has_capability(resolution, RATBAG_RESOLUTION_CAP_DISABLE))
+			return -ENOTSUP;
+
+		dpi = dpi_x = dpi_y = 0;
+	}
+
 	if (drv_data->capabilities & HIDPP_CAP_ONBOARD_PROFILES_8100)
 		return hidpp20drv_update_resolution_dpi_8100(resolution, dpi_x, dpi_y);
 
@@ -1012,22 +985,24 @@ hidpp20drv_update_resolution_dpi(struct ratbag_resolution *resolution,
 	/* just for clarity, we use the first available sensor only */
 	sensor = &drv_data->sensors[0];
 
-	/* validate that the sensor accepts the given DPI */
-	if (dpi < sensor->dpi_min || dpi > sensor->dpi_max)
-		return -EINVAL;
-	if (sensor->dpi_steps) {
-		for (i = sensor->dpi_min; i < dpi; i += sensor->dpi_steps) {
-		}
-		if (i != dpi)
+	if (!resolution->is_disabled) {
+		/* validate that the sensor accepts the given DPI */
+		if (dpi < sensor->dpi_min || dpi > sensor->dpi_max)
 			return -EINVAL;
-	} else {
-		i = 0;
-		while (sensor->dpi_list[i]) {
-			if (sensor->dpi_list[i] == dpi)
-				break;
+		if (sensor->dpi_steps) {
+			for (i = sensor->dpi_min; i < dpi; i += sensor->dpi_steps) {
+			}
+			if (i != dpi)
+				return -EINVAL;
+		} else {
+			i = 0;
+			while (sensor->dpi_list[i]) {
+				if (sensor->dpi_list[i] == dpi)
+					break;
+			}
+			if (sensor->dpi_list[i] != dpi)
+				return -EINVAL;
 		}
-		if (sensor->dpi_list[i] != dpi)
-			return -EINVAL;
 	}
 
 	return hidpp20_adjustable_dpi_set_sensor_dpi(drv_data->dev, sensor, dpi);
@@ -1070,7 +1045,7 @@ hidpp20drv_update_report_rate(struct ratbag_profile *profile, int hz)
 	if (drv_data->capabilities & HIDPP_CAP_ONBOARD_PROFILES_8100)
 		return hidpp20drv_update_report_rate_8100(profile, hz);
 
-	if (drv_data->capabilities & HIDPP_CAP_ADJUSTIBLE_REPORT_RATE_8060) {
+	if (drv_data->capabilities & HIDPP_CAP_ADJUSTABLE_REPORT_RATE_8060) {
 		rc = hidpp20drv_update_report_rate_8060(profile, hz);
 
 		/* re-populate the profile with the correct value if we fail */
@@ -1206,6 +1181,7 @@ hidpp20drv_read_profile_8100(struct ratbag_profile *profile)
 	struct ratbag_resolution *res;
 	struct hidpp20_profile *p;
 	int dpi_index = 0xff;
+	int dpi;
 
 	profile->is_enabled = drv_data->profiles->profiles[profile->index].enabled;
 
@@ -1226,9 +1202,15 @@ hidpp20drv_read_profile_8100(struct ratbag_profile *profile)
 		 * sensors is too niche to care about right now */
 		sensor = &drv_data->sensors[0];
 
-		ratbag_resolution_set_resolution(res,
-						 p->dpi[res->index],
-						 p->dpi[res->index]);
+		dpi = p->dpi[res->index];
+
+		/* If the resolution is zero dpi it is disabled,
+		 * but internally we set the minimum value */
+		if (dpi == 0) {
+			res->is_disabled = true;
+			dpi = sensor->dpi_min;
+		}
+		ratbag_resolution_set_resolution(res, dpi, dpi);
 
 		if (profile->is_active &&
 		    res->index == (unsigned int)dpi_index)
@@ -1247,7 +1229,7 @@ hidpp20drv_read_profile_8100(struct ratbag_profile *profile)
 	ratbag_profile_set_report_rate_list(profile,
 					    drv_data->report_rates,
 					    drv_data->num_report_rates);
-	ratbag_profile_set_report_rate(profile, p->report_rate);
+	profile->hz = p->report_rate;
 }
 
 static int
@@ -1309,12 +1291,17 @@ hidpp20drv_read_profile(struct ratbag_profile *profile)
 {
 	struct ratbag_device *device = profile->device;
 	struct hidpp20drv_data *drv_data = ratbag_get_drv_data(device);
+	struct ratbag_resolution *resolution;
 	struct ratbag_led *led;
 	struct ratbag_button *button;
 
 	if (drv_data->capabilities & HIDPP_CAP_ONBOARD_PROFILES_8100) {
 		hidpp20drv_read_profile_8100(profile);
 		ratbag_profile_set_cap(profile, RATBAG_PROFILE_CAP_DISABLE);
+
+		ratbag_profile_for_each_resolution(profile, resolution) {
+			ratbag_resolution_set_cap(resolution, RATBAG_RESOLUTION_CAP_DISABLE);
+		}
 	} else {
 		hidpp20drv_read_resolution_dpi(profile);
 		hidpp20drv_read_special_key_mouse(device);
@@ -1427,11 +1414,11 @@ hidpp20drv_init_feature(struct ratbag_device *device, uint16_t feature)
 		if (rc < 0)
 			return 0; /* this is not a hard failure */
 
-		drv_data->capabilities |= HIDPP_CAP_ADJUSTIBLE_REPORT_RATE_8060;
+		drv_data->capabilities |= HIDPP_CAP_ADJUSTABLE_REPORT_RATE_8060;
 		break;
 	}
 	case HIDPP_PAGE_COLOR_LED_EFFECTS: {
-		/* The 8070 feauture implemented in the G602 doesn't follow the spec,
+		/* The 8070 feature implemented in the G602 doesn't follow the spec,
 		 * so we ignore it */
 		if (ratbag_device_data_hidpp20_get_quirk(device->data) == HIDPP20_QUIRK_G602)
 			break;
