@@ -37,7 +37,7 @@
 #define LOGITECH_G300_BUTTON_MAX			8
 #define LOGITECH_G300_NUM_DPI				4
 #define LOGITECH_G300_NUM_LED				1
-#define LOGITECH_G300_DPI_MIN				100
+#define LOGITECH_G300_DPI_MIN				250
 #define LOGITECH_G300_DPI_MAX				2500
 
 #define LOGITECH_G300_REPORT_ID_GET_ACTIVE		0xF0
@@ -51,7 +51,7 @@
 #define LOGITECH_G300_REPORT_SIZE_PROFILE		35
 
 struct logitech_g300_resolution {
-	uint8_t dpi :7; /* Range 1-10. dpi = 100*value */
+	uint8_t dpi :7; /* Range 1-10. dpi = 250*value */
 	uint8_t is_default :1;
 } __attribute__((packed));
 
@@ -84,43 +84,13 @@ struct logitech_g300_data {
 _Static_assert(sizeof(struct logitech_g300_profile_report) == LOGITECH_G300_REPORT_SIZE_PROFILE,
 	       "Size of logitech_g300_profile_report is wrong");
 
-struct logitech_g300_button_type_mapping {
-	uint8_t raw;
-	enum ratbag_button_type type;
-};
-
-static const struct logitech_g300_button_type_mapping logitech_g300_button_type_mapping[] = {
-	{ 0, RATBAG_BUTTON_TYPE_LEFT },
-	{ 1, RATBAG_BUTTON_TYPE_RIGHT },
-	{ 2, RATBAG_BUTTON_TYPE_MIDDLE },
-	{ 3, RATBAG_BUTTON_TYPE_THUMB },
-	{ 4, RATBAG_BUTTON_TYPE_THUMB2 },
-	{ 5, RATBAG_BUTTON_TYPE_PINKIE },
-	{ 6, RATBAG_BUTTON_TYPE_PINKIE2 },
-	{ 7, RATBAG_BUTTON_TYPE_PROFILE_CYCLE_UP },
-	{ 8, RATBAG_BUTTON_TYPE_RESOLUTION_CYCLE_UP },
-};
-
-static enum ratbag_button_type
-logitech_g300_raw_to_button_type(uint8_t data)
-{
-	const struct logitech_g300_button_type_mapping *mapping;
-
-	ARRAY_FOR_EACH(logitech_g300_button_type_mapping, mapping) {
-		if (mapping->raw == data)
-			return mapping->type;
-	}
-
-	return RATBAG_BUTTON_TYPE_UNKNOWN;
-}
-
 struct logitech_g300_button_mapping {
 	uint8_t raw;
 	struct ratbag_button_action action;
 };
 
 static struct logitech_g300_button_mapping logitech_g300_button_mapping[] = {
-	/* 0x00 is either key or unassigned. Must be handled separatly  */
+	/* 0x00 is either key or unassigned. Must be handled separately  */
 	{ 0x01, BUTTON_ACTION_BUTTON(1) },
 	{ 0x02, BUTTON_ACTION_BUTTON(2) },
 	{ 0x03, BUTTON_ACTION_BUTTON(3) },
@@ -352,13 +322,11 @@ logitech_g300_read_button(struct ratbag_button *button)
 	profile_report = &pdata->report;
 	button_report = &profile_report->buttons[button->index];
 
+	ratbag_button_enable_action_type(button, RATBAG_BUTTON_ACTION_TYPE_NONE);
 	ratbag_button_enable_action_type(button, RATBAG_BUTTON_ACTION_TYPE_BUTTON);
 	ratbag_button_enable_action_type(button, RATBAG_BUTTON_ACTION_TYPE_SPECIAL);
-	ratbag_button_enable_action_type(button, RATBAG_BUTTON_ACTION_TYPE_KEY);
 	ratbag_button_enable_action_type(button, RATBAG_BUTTON_ACTION_TYPE_MACRO);
 
-	button->type = logitech_g300_raw_to_button_type(button->index);
-
 	action = logitech_g300_raw_to_button_action(button_report->code);
 	if (action) {
 		ratbag_button_set_action(button, action);
@@ -394,7 +362,6 @@ logitech_g300_read_led(struct ratbag_led *led)
 	pdata = &drv_data->profile_data[profile->index];
 	profile_report = &pdata->report;
 
-	led->type = RATBAG_LED_TYPE_SIDE;
 	led->mode = RATBAG_LED_ON;
 	led->colordepth = RATBAG_LED_COLORDEPTH_RGB_111;
 	led->color.red = profile_report->led_red * 255;
@@ -427,6 +394,10 @@ logitech_g300_read_profile(struct ratbag_profile *profile)
 	case 0: report_id = LOGITECH_G300_REPORT_ID_PROFILE_0; break;
 	case 1: report_id = LOGITECH_G300_REPORT_ID_PROFILE_1; break;
 	case 2: report_id = LOGITECH_G300_REPORT_ID_PROFILE_2; break;
+	default:
+		/* Should've been handled by the assertion above. */
+		abort();
+		break;
 	}
 
 	rc = ratbag_hidraw_raw_request(device, report_id,
@@ -444,15 +415,16 @@ logitech_g300_read_profile(struct ratbag_profile *profile)
 	hz = logitech_g300_raw_to_frequency(report->frequency);
 
 	ratbag_profile_set_report_rate_list(profile, &hz, 1);
-	ratbag_profile_set_report_rate(profile, hz);
+	profile->hz = hz;
 
 	ratbag_profile_for_each_resolution(profile, resolution) {
 		struct logitech_g300_resolution *res =
 			&report->dpi_levels[resolution->index];
-		unsigned int dpis[49] = { 100, 150, 200, 250, 300, 350, 400, 450, 500, 550, 600, 650, 700, 750, 800, 850, 900, 950, 1000, 1050, 1100, 1150, 1200, 1250, 1300, 1350, 1400, 1450, 1500, 1550, 1600, 1650, 1700, 1750, 1800, 1850, 1900, 1950, 2000, 2050, 2100, 2150, 2200, 2250, 2300, 2350, 2400, 2450, 2500, };
+		unsigned int dpis[10] = { 250, 500, 750, 1000, 1250, 1500, 1750,
+					  2000, 2250, 2500 };
 
-		resolution->dpi_x = res->dpi * 50;
-		resolution->dpi_y = res->dpi * 50;
+		resolution->dpi_x = res->dpi * 250;
+		resolution->dpi_y = res->dpi * 250;
 		resolution->is_default = res->is_default;
 		resolution->is_active = res->is_default;
 
@@ -545,7 +517,7 @@ logitech_g300_write_profile(struct ratbag_profile *profile)
 		struct logitech_g300_resolution *res =
 			&report->dpi_levels[resolution->index];
 
-		res->dpi = resolution->dpi_x / 50;
+		res->dpi = resolution->dpi_x / 250;
 		res->is_default = resolution->is_default;
 
 		if (profile->is_active && resolution->is_active)
