@@ -168,51 +168,6 @@ struct roccat_data {
 	struct roccat_macro macros[(ROCCAT_PROFILE_MAX)][(ROCCAT_BUTTON_MAX + 1)];
 };
 
-struct roccat_button_type_mapping {
-	uint8_t raw;
-	enum ratbag_button_type type;
-};
-
-static const struct roccat_button_type_mapping roccat_button_type_mapping[] = {
-	{ 0, RATBAG_BUTTON_TYPE_LEFT },
-	{ 1, RATBAG_BUTTON_TYPE_RIGHT },
-	{ 2, RATBAG_BUTTON_TYPE_MIDDLE },
-	{ 3, RATBAG_BUTTON_TYPE_THUMB },                // Up
-	{ 4, RATBAG_BUTTON_TYPE_THUMB2 },               // Down
-	{ 5, RATBAG_BUTTON_TYPE_WHEEL_LEFT },
-	{ 6, RATBAG_BUTTON_TYPE_WHEEL_RIGHT },
-	{ 7, RATBAG_BUTTON_TYPE_WHEEL_UP },
-	{ 8, RATBAG_BUTTON_TYPE_WHEEL_DOWN },
-	{ 9, RATBAG_BUTTON_TYPE_RESOLUTION_UP },
-	{ 10, RATBAG_BUTTON_TYPE_RESOLUTION_DOWN },
-
-	// Easy shift+, these buttons are not physical
-	{ 11, RATBAG_BUTTON_TYPE_LEFT },
-	{ 12, RATBAG_BUTTON_TYPE_RIGHT },
-	{ 13, RATBAG_BUTTON_TYPE_MIDDLE },
-	{ 14, RATBAG_BUTTON_TYPE_THUMB },                // Up
-	{ 15, RATBAG_BUTTON_TYPE_THUMB2 },               // Down
-	{ 16, RATBAG_BUTTON_TYPE_WHEEL_LEFT },
-	{ 17, RATBAG_BUTTON_TYPE_WHEEL_RIGHT },
-	{ 18, RATBAG_BUTTON_TYPE_WHEEL_UP },
-	{ 19, RATBAG_BUTTON_TYPE_WHEEL_DOWN },
-	{ 20, RATBAG_BUTTON_TYPE_RESOLUTION_UP },
-	{ 21, RATBAG_BUTTON_TYPE_RESOLUTION_DOWN },
-};
-
-static enum ratbag_button_type
-roccat_raw_to_button_type(uint8_t data)
-{
-	const struct roccat_button_type_mapping *mapping;
-
-	ARRAY_FOR_EACH(roccat_button_type_mapping, mapping) {
-		if (mapping->raw == data)
-			return mapping->type;
-	}
-
-	return RATBAG_BUTTON_TYPE_UNKNOWN;
-}
-
 struct roccat_button_mapping {
 	uint8_t raw;
 	struct ratbag_button_action action;
@@ -238,7 +193,7 @@ static struct roccat_button_mapping roccat_button_mapping[] = {
 	{ 20, BUTTON_ACTION_SPECIAL(RATBAG_BUTTON_ACTION_SPECIAL_RESOLUTION_CYCLE_UP) },
 	{ 21, BUTTON_ACTION_SPECIAL(RATBAG_BUTTON_ACTION_SPECIAL_RESOLUTION_UP) },
 	{ 22, BUTTON_ACTION_SPECIAL(RATBAG_BUTTON_ACTION_SPECIAL_RESOLUTION_DOWN) },
-/* FIXME:   { 23, Toogle sensibility }, */
+/* FIXME:   { 23, Toggle sensibility }, */
 /* FIXME:   { 24, Sensibility UP }, */
 /* FIXME:   { 25, Sensibility Down }, */
 /* FIXME:   { 27, open driver/swarm },  -> hidraw report 02 83 01 00 00 00 00 00 */
@@ -520,7 +475,7 @@ roccat_set_current_profile(struct ratbag_device *device, unsigned int index)
  * Sets the profile and which information we want to get from the mouse
  *
  * @param profile is the index of the profile from which you want the info. But, it is also used as a memory bank
- * identifier when quering a macro. In this case, the first bank can be queried by adding 0x10 to the profile index,
+ * identifier when querying a macro. In this case, the first bank can be queried by adding 0x10 to the profile index,
  * and the second bank, by adding 0x20.
  * @param type can be either which information you need (ROCCAT_CONFIG_SETTINGS or ROCCAT_CONFIG_KEY_MAPPING) or
  * it can be used to specify the button from which you want to get the macro.
@@ -836,14 +791,12 @@ roccat_read_button(struct ratbag_button *button)
 	action = roccat_button_to_action(button->profile, button->index);
 	if (action)
 		ratbag_button_set_action(button, action);
-	button->type = roccat_raw_to_button_type(button->index);
 //  if (action == NULL)
 //      log_error(device->ratbag, "button: %d -> %d %s:%d\n",
 //          button->index, drv_data->profiles[button->profile->index][3 + button->index * 3],
 //          __FILE__, __LINE__);
 
 	ratbag_button_enable_action_type(button, RATBAG_BUTTON_ACTION_TYPE_BUTTON);
-	ratbag_button_enable_action_type(button, RATBAG_BUTTON_ACTION_TYPE_KEY);
 	ratbag_button_enable_action_type(button, RATBAG_BUTTON_ACTION_TYPE_SPECIAL);
 	ratbag_button_enable_action_type(button, RATBAG_BUTTON_ACTION_TYPE_MACRO);
 
@@ -918,7 +871,7 @@ roccat_read_dpi(struct roccat_settings_report* settings, struct ratbag_profile*
 
 	ratbag_profile_set_report_rate_list(profile, report_rates,
 						ARRAY_LENGTH(report_rates));
-	ratbag_profile_set_report_rate(profile, report_rate);
+	profile->hz = report_rate;
 
 	ratbag_profile_for_each_resolution(profile, resolution) {
 		dpi_x = settings->xres[resolution->index] * 100 + 100;
@@ -941,7 +894,6 @@ roccat_read_dpi(struct roccat_settings_report* settings, struct ratbag_profile*
 static void
 roccat_read_led(struct roccat_settings_report* settings, struct ratbag_led *led)
 {
-	led->type = RATBAG_LED_TYPE_SIDE;
 	if(settings->led_status == 0) {
 		led->mode = RATBAG_LED_OFF;
 	} else {
