@@ -34,7 +34,6 @@
 #include "libratbag-private.h"
 #include "libratbag-hidraw.h"
 #include "libratbag-data.h"
-#include "hidpp-generic.h"
 
 #define STEELSERIES_NUM_PROFILES	1
 #define STEELSERIES_NUM_DPI		2
@@ -65,14 +64,17 @@
 #define STEELSERIES_ID_LED			0x5b
 #define STEELSERIES_ID_SAVE			0x59
 #define STEELSERIES_ID_FIRMWARE_PROTOCOL2	0x90
-#define STEELSERIES_ID_SETTTINGS		0x92
+#define STEELSERIES_ID_SETTINGS		0x92
 
 #define STEELSERIES_ID_DPI_PROTOCOL3		0x03
 #define STEELSERIES_ID_REPORT_RATE_PROTOCOL3	0x04
 #define STEELSERIES_ID_LED_PROTOCOL3		0x05
 #define STEELSERIES_ID_SAVE_PROTOCOL3		0x09
 #define STEELSERIES_ID_FIRMWARE_PROTOCOL3	0x10
-#define STEELSERIES_ID_SETTTINGS_PROTOCOL3	0x16
+#define STEELSERIES_ID_SETTINGS_PROTOCOL3	0x16
+
+#define STEELSERIES_ID_DPI_PROTOCOL4		0x15
+#define STEELSERIES_ID_REPORT_RATE_PROTOCOL4	0x17
 
 #define STEELSERIES_BUTTON_OFF			0x00
 #define STEELSERIES_BUTTON_RES_CYCLE		0x30
@@ -82,11 +84,6 @@
 #define STEELSERIES_BUTTON_KBD			0x51
 #define STEELSERIES_BUTTON_CONSUMER		0x61
 
-struct steelseries_data {
-	int firmware_major;
-	int firmware_minor;
-};
-
 struct steelseries_point {
 	struct list link;
 
@@ -103,7 +100,7 @@ struct steelseries_led_cycle {
 };
 
 struct steelseries_led_cycle_spec {
-	int hid_report_type;	/* either HID_OUTPUT_REPORT or HID_FEATURE_REPORT */
+	uint8_t hid_report_type;/* either HID_OUTPUT_REPORT or HID_FEATURE_REPORT */
 	int header_len;		/* number of bytes in the header */
 	uint8_t cmd_val;	/* command value for the color command */
 	bool has_2_led_ids;	/* some mice have 2 fields for led id */
@@ -116,13 +113,11 @@ struct steelseries_led_cycle_spec {
 	int point_count_idx;	/* index of the point counter field */
 };
 
-struct _steelseries_message {
-	uint8_t report_id;
-	uint8_t parameters[STEELSERIES_REPORT_SIZE - 1];
-} __attribute__((packed));
-
 union steelseries_message {
-	struct _steelseries_message msg;
+	struct {
+		uint8_t report_id;
+		uint8_t parameters[STEELSERIES_REPORT_SIZE - 1];
+	} __attribute__((packed)) msg;
 	uint8_t data[STEELSERIES_REPORT_SIZE];
 };
 
@@ -137,22 +132,27 @@ steelseries_test_hidraw(struct ratbag_device *device)
 	/* Rival mice are composite devices with multiple HID devices
 	 * and only the HID vendor device can be used to configure the
 	 * device.
+	 * However, this check doesn't apply to some devices, like
+	 * Sensei 310, Rival 600, Rival 650, etc, so we can't rely on
+	 * it.
 	 */
-	if (!ratbag_hidraw_has_vendor_page(device))
-		return false;
+	if (!ratbag_hidraw_has_vendor_page(device)) {
+		log_debug(device->ratbag,
+			  "This is a non-vendor HID device, "
+			  "it may show up as a duplicate configurable device in libratbag\n");
+	}
 
 	if (device_version > 1)
 		return ratbag_hidraw_has_report(device, STEELSERIES_REPORT_ID_1);
-	else
-		return true;
+
+	return true;
 }
 
 static void
-button_defaults_for_layout(struct ratbag_button *button, int button_count)
+button_defaults_for_layout(struct ratbag_button *button, size_t button_count)
 {
 	/* The default button mapping vary depending on the number of buttons
 	 * on the device. */
-	enum ratbag_button_type button_types[8] = {RATBAG_BUTTON_TYPE_UNKNOWN};
 	struct ratbag_button_action button_actions[8] = {
 		BUTTON_ACTION_BUTTON(1),
 		BUTTON_ACTION_BUTTON(2),
@@ -178,30 +178,12 @@ button_defaults_for_layout(struct ratbag_button *button, int button_count)
 		button_actions[7].action.special = RATBAG_BUTTON_ACTION_SPECIAL_RESOLUTION_CYCLE_UP;
 	}
 
-	button_types[0] = RATBAG_BUTTON_TYPE_LEFT;
-	button_types[1] = RATBAG_BUTTON_TYPE_RIGHT;
-	button_types[2] = RATBAG_BUTTON_TYPE_MIDDLE;
-	button_types[3] = RATBAG_BUTTON_TYPE_THUMB;
-	button_types[4] = RATBAG_BUTTON_TYPE_THUMB2;
-	if (button_count <= 6) {
-		button_types[5] = RATBAG_BUTTON_TYPE_RESOLUTION_CYCLE_UP;
-	} else if (button_count == 7) {
-		button_types[5] = RATBAG_BUTTON_TYPE_THUMB3;
-		button_types[6] = RATBAG_BUTTON_TYPE_RESOLUTION_CYCLE_UP;
-	} else {
-		button_types[5] = RATBAG_BUTTON_TYPE_PINKIE;
-		button_types[6] = RATBAG_BUTTON_TYPE_PINKIE2;
-		button_types[7] = RATBAG_BUTTON_TYPE_RESOLUTION_CYCLE_UP;
-	}
-
-	button->type = button_types[button->index];
 	ratbag_button_set_action(button, &button_actions[button->index]);
 }
 
 static int
-steelseries_get_firmware_version(struct ratbag_device *device)
+steelseries_get_firmware_version(struct ratbag_device *device, int *major_out, int *minor_out)
 {
-	struct steelseries_data *drv_data = device->drv_data;
 	int device_version = ratbag_device_data_steelseries_get_device_version(device->data);
 	size_t msg_len;
 	uint8_t buf[2] = {0};
@@ -225,6 +207,7 @@ steelseries_get_firmware_version(struct ratbag_device *device)
 		msg.msg.parameters[0] = STEELSERIES_ID_FIRMWARE_PROTOCOL3;
 		msg_len = STEELSERIES_REPORT_SIZE;
 		break;
+	case 4:
 	default:
 		return -ENOTSUP;
 	}
@@ -238,8 +221,11 @@ steelseries_get_firmware_version(struct ratbag_device *device)
 	if (ret < 0)
 		return ret;
 
-	drv_data->firmware_major = buf[1];
-	drv_data->firmware_minor = buf[0];
+	// TODO: check if these are in correct order.
+	// Rivalcfg - another configuration utility for SteelSeries mice -
+	// was updated to invert their order on 2022-08-26.
+	*minor_out = buf[0];
+	*major_out = buf[1];
 
 	return 0;
 }
@@ -263,12 +249,11 @@ steelseries_read_settings(struct ratbag_device *device)
 
 	switch (device_version) {
 	case 2:
-		msg.msg.parameters[0] = STEELSERIES_ID_SETTTINGS;
+		msg.msg.parameters[0] = STEELSERIES_ID_SETTINGS;
 		break;
 	case 3:
-		msg.msg.parameters[0] = STEELSERIES_ID_SETTTINGS_PROTOCOL3;
+		msg.msg.parameters[0] = STEELSERIES_ID_SETTINGS_PROTOCOL3;
 		break;
-	case 1:
 	default:
 		return -ENOTSUP;
 	}
@@ -313,16 +298,14 @@ steelseries_read_settings(struct ratbag_device *device)
 static int
 steelseries_probe(struct ratbag_device *device)
 {
-	struct steelseries_data *drv_data = NULL;
 	struct ratbag_profile *profile = NULL;
 	struct ratbag_resolution *resolution;
 	struct ratbag_button *button;
 	struct ratbag_led *led;
-	int rc, button_count, led_count, device_version, quirk;
-	struct dpi_list *dpilist = NULL;
-	struct dpi_range *dpirange = NULL;
+	int device_version;
+	int rc;
 
-	unsigned int report_rates[] = { 125, 250, 500, 1000 };
+	static const unsigned int report_rates[] = { 125, 250, 500, 1000 };
 
 	rc = ratbag_find_hidraw(device, steelseries_test_hidraw);
 	if (rc)
@@ -340,17 +323,19 @@ steelseries_probe(struct ratbag_device *device)
 		return -EINVAL;
 	}
 
-	button_count = ratbag_device_data_steelseries_get_button_count(device->data);
-	if (button_count == -1)
-		button_count = 0;
+	rc = ratbag_device_data_steelseries_get_button_count(device->data);
+	if (rc == -1)
+		rc = 0;
+	const unsigned int button_count = (unsigned int)rc;
 
-	led_count = ratbag_device_data_steelseries_get_led_count(device->data);
-	if (led_count == -1)
-		led_count = 0;
+	rc = ratbag_device_data_steelseries_get_led_count(device->data);
+	if (rc == -1)
+		rc = 0;
+	const unsigned int led_count = (unsigned int)rc;
 
-	dpirange = ratbag_device_data_steelseries_get_dpi_range(device->data);
-	dpilist = ratbag_device_data_steelseries_get_dpi_list(device->data);
-	quirk = ratbag_device_data_steelseries_get_quirk(device->data);
+	const enum steelseries_quirk quirk = ratbag_device_data_steelseries_get_quirk(device->data);
+	const struct dpi_list *dpilist = ratbag_device_data_steelseries_get_dpi_list(device->data);
+	const struct dpi_range *dpirange = ratbag_device_data_steelseries_get_dpi_range(device->data);
 
 	ratbag_device_init_profiles(device,
 				    STEELSERIES_NUM_PROFILES,
@@ -367,7 +352,7 @@ steelseries_probe(struct ratbag_device *device)
 		ratbag_profile_set_cap(profile, RATBAG_PROFILE_CAP_WRITE_ONLY);
 		ratbag_profile_set_report_rate_list(profile, report_rates,
 						    ARRAY_LENGTH(report_rates));
-		ratbag_profile_set_report_rate(profile, 1000);
+		profile->hz = 1000;
 
 		ratbag_profile_for_each_resolution(profile, resolution) {
 			if (resolution->index == 0) {
@@ -393,7 +378,6 @@ steelseries_probe(struct ratbag_device *device)
 		ratbag_profile_for_each_button(profile, button) {
 			ratbag_button_enable_action_type(button, RATBAG_BUTTON_ACTION_TYPE_BUTTON);
 			ratbag_button_enable_action_type(button, RATBAG_BUTTON_ACTION_TYPE_SPECIAL);
-			ratbag_button_enable_action_type(button, RATBAG_BUTTON_ACTION_TYPE_KEY);
 			if (quirk != STEELSERIES_QUIRK_SENSEIRAW) {
 				ratbag_button_enable_action_type(button, RATBAG_BUTTON_ACTION_TYPE_MACRO);
 			}
@@ -402,7 +386,6 @@ steelseries_probe(struct ratbag_device *device)
 		}
 
 		ratbag_profile_for_each_led(profile, led) {
-			led->type = led->index == 0 ? RATBAG_LED_TYPE_LOGO : RATBAG_LED_TYPE_WHEEL;
 			led->mode = RATBAG_LED_ON;
 			if (quirk == STEELSERIES_QUIRK_SENSEIRAW) {
 				led->colordepth = RATBAG_LED_COLORDEPTH_MONOCHROME;
@@ -421,16 +404,22 @@ steelseries_probe(struct ratbag_device *device)
 		}
 	}
 
-	drv_data = zalloc(sizeof(*drv_data));
-	ratbag_set_drv_data(device, drv_data);
+	{
+		int firmware_minor = 0;
+		int firmware_major = 0;
+		rc = steelseries_get_firmware_version(device, &firmware_major, &firmware_minor);
+		if (rc == 0) {
+			_cleanup_free_ char *fw = asprintf_safe("%d.%d", firmware_major, firmware_minor);
+			ratbag_device_set_firmware_version(device, fw);
+		}
+	}
 
-	rc = steelseries_get_firmware_version(device);
-	if(rc == 0)
-		log_debug(device->ratbag, "SteelSeries firmware version %d.%d\n",
-			drv_data->firmware_major,
-			drv_data->firmware_minor);
-
-	steelseries_read_settings(device);
+	rc = steelseries_read_settings(device);
+	/* Some devices don't support reading settings, so ignore ENOTSUP. */
+	if (rc < 0 && rc != -ENOTSUP) {
+		log_error(device->ratbag, "Failed to read device settings\n");
+		return rc;
+	}
 
 	return 0;
 }
@@ -443,7 +432,6 @@ steelseries_write_dpi(struct ratbag_resolution *resolution)
 	struct dpi_list *dpilist = NULL;
 	struct dpi_range *dpirange = NULL;
 	int ret;
-	int i = 0;
 	size_t buf_len;
 
 	union steelseries_message msg = {
@@ -455,37 +443,45 @@ steelseries_write_dpi(struct ratbag_resolution *resolution)
 	dpilist = ratbag_device_data_steelseries_get_dpi_list(device->data);
 
 	switch (device_version) {
-	case 1:
+	case 1: {
+		size_t i = 0;
 		/* when using lists the entries are enumerated in reverse */
 		if (dpilist) {
-			for (i = 0; i < (int)resolution->ndpis; i++) {
+			for (i = 0; i < resolution->ndpis; i++) {
 				if (resolution->dpis[i] == resolution->dpi_x)
 					break;
 			}
 			i = resolution->ndpis - i;
 		} else {
-			i = resolution->dpi_x / dpirange->step - 1;
+			i = resolution->dpi_x / (size_t)dpirange->step - 1U;
 		}
 
 		buf_len = STEELSERIES_REPORT_SIZE_SHORT;
 		msg.msg.parameters[0] = STEELSERIES_ID_DPI_SHORT;
-		msg.msg.parameters[1] = resolution->index + 1;
-		msg.msg.parameters[2] = i;
+		msg.msg.parameters[1] = (uint8_t)resolution->index + 1;
+		msg.msg.parameters[2] = (uint8_t)i;
 		break;
+	}
 	case 2:
 		buf_len = STEELSERIES_REPORT_SIZE;
 		msg.msg.parameters[0] = STEELSERIES_ID_DPI;
-		msg.msg.parameters[2] = resolution->index + 1;
-		msg.msg.parameters[3] = resolution->dpi_x / dpirange->step - 1;
+		msg.msg.parameters[2] = (uint8_t)resolution->index + 1;
+		msg.msg.parameters[3] = (uint8_t)(resolution->dpi_x / (size_t)dpirange->step - 1);
 		msg.msg.parameters[6] = 0x42; /* not sure if needed */
 		break;
 	case 3:
 		buf_len = STEELSERIES_REPORT_SIZE;
 		msg.msg.parameters[0] = STEELSERIES_ID_DPI_PROTOCOL3;
-		msg.msg.parameters[2] = resolution->index + 1;
-		msg.msg.parameters[3] = resolution->dpi_x / dpirange->step - 1;
+		msg.msg.parameters[2] = (uint8_t)resolution->index + 1;
+		msg.msg.parameters[3] = (uint8_t)(resolution->dpi_x / (size_t)dpirange->step - 1);
 		msg.msg.parameters[5] = 0x42; /* not sure if needed */
 		break;
+	case 4:
+		buf_len = STEELSERIES_REPORT_SIZE;
+		msg.msg.parameters[0] = STEELSERIES_ID_DPI_PROTOCOL4;
+		msg.msg.parameters[1] = (uint8_t)resolution->index + 1;
+		msg.msg.parameters[2] = (uint8_t)(resolution->dpi_x / (size_t)dpirange->step - 1);
+		break;
 	default:
 		return -ENOTSUP;
 	}
@@ -513,6 +509,7 @@ steelseries_write_report_rate(struct ratbag_profile *profile)
 	};
 
 	switch (device_version) {
+	case 4:
 	case 1:
 		if (profile->hz >= 1000) {
 			profile->hz = 1000;
@@ -529,18 +526,18 @@ steelseries_write_report_rate(struct ratbag_profile *profile)
 		}
 
 		buf_len = STEELSERIES_REPORT_SIZE_SHORT;
-		msg.msg.parameters[0] = STEELSERIES_ID_REPORT_RATE_SHORT;
+		msg.msg.parameters[0] = device_version == 1 ? STEELSERIES_ID_REPORT_RATE_SHORT : STEELSERIES_ID_REPORT_RATE_PROTOCOL4;
 		msg.msg.parameters[2] = reported_rate;
 		break;
 	case 2:
 		buf_len = STEELSERIES_REPORT_SIZE;
 		msg.msg.parameters[0] = STEELSERIES_ID_REPORT_RATE;
-		msg.msg.parameters[2] = 1000 / profile->hz;
+		msg.msg.parameters[2] = (uint8_t)1000 / profile->hz;
 		break;
         case 3:
 		buf_len = STEELSERIES_REPORT_SIZE;
 		msg.msg.parameters[0] = STEELSERIES_ID_REPORT_RATE_PROTOCOL3;
-		msg.msg.parameters[2] = 1000 / profile->hz;
+		msg.msg.parameters[2] = (uint8_t)1000 / profile->hz;
 		break;
 	default:
 		return -ENOTSUP;
@@ -566,8 +563,8 @@ steelseries_write_buttons(struct ratbag_profile *profile)
 		return 0;
 
 	const bool is_senseiraw = ratbag_device_data_steelseries_get_quirk(device->data) == STEELSERIES_QUIRK_SENSEIRAW;
-	const int button_size = is_senseiraw ? 3 : 5;
-	const int report_size = is_senseiraw ? STEELSERIES_REPORT_SIZE_SHORT : STEELSERIES_REPORT_LONG_SIZE;
+	const size_t button_size = is_senseiraw ? 3 : 5;
+	const size_t report_size = is_senseiraw ? STEELSERIES_REPORT_SIZE_SHORT : STEELSERIES_REPORT_LONG_SIZE;
 	const int max_modifiers = is_senseiraw ? 0 : 3;
 
 	union steelseries_message msg = {
@@ -579,16 +576,15 @@ steelseries_write_buttons(struct ratbag_profile *profile)
 
 	ratbag_profile_for_each_button(profile, button) {
 		struct ratbag_button_action *action = &button->action;
-		uint16_t code;
-		unsigned int key, modifiers;
-		int idx;
+		unsigned int key;
+		unsigned int modifiers;
 
 		/* Each button takes up 3 or 5 bytes starting from index 2 */
-		idx = 2 + button->index * button_size;
+		size_t idx = 2 + button->index * button_size;
 
 		switch (action->type) {
 		case RATBAG_BUTTON_ACTION_TYPE_BUTTON:
-			msg.msg.parameters[idx] = action->action.button;
+			msg.msg.parameters[idx] = (uint8_t)action->action.button;
 			break;
 
 		case RATBAG_BUTTON_ACTION_TYPE_MACRO:
@@ -602,9 +598,9 @@ steelseries_write_buttons(struct ratbag_profile *profile)
 				break;
 			}
 
-			code = ratbag_hidraw_get_keyboard_usage_from_keycode(
+			const uint8_t keyboard_code = ratbag_hidraw_get_keyboard_usage_from_keycode(
 						device, key);
-			if (code) {
+			if (keyboard_code) {
 				if (is_senseiraw) {
 					msg.msg.parameters[idx] = STEELSERIES_BUTTON_KEY;
 				} else {
@@ -628,12 +624,12 @@ steelseries_write_buttons(struct ratbag_profile *profile)
 						msg.msg.parameters[++idx] = 0xE7;
 				}
 
-				msg.msg.parameters[idx + 1] = code;
+				msg.msg.parameters[idx + 1] = keyboard_code;
 			} else {
-				code = ratbag_hidraw_get_consumer_usage_from_keycode(
-						device, action->action.key.key);
+				const uint16_t consumer_code = ratbag_hidraw_get_consumer_usage_from_keycode(
+						device, action->action.key);
 				msg.msg.parameters[idx] = STEELSERIES_BUTTON_CONSUMER;
-				msg.msg.parameters[idx + 1] = code;
+				msg.msg.parameters[idx + 1] = (uint8_t)consumer_code;
 			}
 			break;
 
@@ -653,7 +649,6 @@ steelseries_write_buttons(struct ratbag_profile *profile)
 			}
 			break;
 
-		case RATBAG_BUTTON_ACTION_TYPE_KEY:
 		case RATBAG_BUTTON_ACTION_TYPE_NONE:
 		default:
 			msg.msg.parameters[idx] = STEELSERIES_BUTTON_OFF;
@@ -688,7 +683,7 @@ steelseries_write_led_v1(struct ratbag_led *led)
 	};
 
 	msg.msg.parameters[0] = STEELSERIES_ID_LED_EFFECT_SHORT;
-	msg.msg.parameters[1] = quirk == STEELSERIES_QUIRK_RIVAL100 ? 0x00 : led->index + 1;
+	msg.msg.parameters[1] = quirk == STEELSERIES_QUIRK_RIVAL100 ? 0x00 : (uint8_t)led->index + 1;
 
 	switch(led->mode) {
 	case RATBAG_LED_OFF:
@@ -723,7 +718,7 @@ steelseries_write_led_v1(struct ratbag_led *led)
 
 	if (quirk == STEELSERIES_QUIRK_SENSEIRAW) {
 		msg.msg.parameters[0] = STEELSERIES_ID_LED_INTENSITY_SHORT;
-		msg.msg.parameters[1] = led->index + 1;
+		msg.msg.parameters[1] = (uint8_t)led->index + 1;
 		if (led->mode == RATBAG_LED_OFF || led->brightness == 0) {
 			msg.msg.parameters[2] = 1;
 		} else {
@@ -733,7 +728,7 @@ steelseries_write_led_v1(struct ratbag_led *led)
 	} else {
 		if (quirk != STEELSERIES_QUIRK_RIVAL100) {
 			msg.msg.parameters[0] = STEELSERIES_ID_LED_COLOR_SHORT;
-			msg.msg.parameters[1] = led->index + 1;
+			msg.msg.parameters[1] = (uint8_t)led->index + 1;
 		} else {
 			msg.msg.parameters[0] = STEELSERIES_ID_LED_COLOR_SHORT_RIVAL100;
 			msg.msg.parameters[1] = 0x00;
@@ -810,7 +805,7 @@ construct_cycle_buffer(struct steelseries_led_cycle *cycle,
 	buf[spec->point_count_idx] = npoints;
 
 	/* this seems to be the minimum allowed */
-	duration = max(buf[spec->point_count_idx] * 330, cycle->duration);
+	duration = max((uint16_t)(buf[spec->point_count_idx] * 330), cycle->duration);
 
 	set_unaligned_le_u16(&buf[spec->duration_idx], duration);
 }
@@ -837,7 +832,7 @@ steelseries_write_led_cycle(struct ratbag_led *led,
 	const struct ratbag_color blue = { 0x00, 0x00, 0xFF };
 
 	create_cycle(&cycle);
-	cycle.led_id = led->index;
+	cycle.led_id = (uint8_t)led->index;
 
 	switch(led->mode) {
 	case RATBAG_LED_OFF:
@@ -874,7 +869,7 @@ steelseries_write_led_cycle(struct ratbag_led *led,
 		list_append(&cycle.points, &point[2].link);
 		list_append(&cycle.points, &point[3].link);
 
-		cycle.duration = led->ms;
+		cycle.duration = (uint16_t)led->ms;
 		break;
 	case RATBAG_LED_BREATHING:
 		point[0].color = black;
@@ -890,7 +885,7 @@ steelseries_write_led_cycle(struct ratbag_led *led,
 		list_append(&cycle.points, &point[1].link);
 		list_append(&cycle.points, &point[2].link);
 
-		cycle.duration = led->ms;
+		cycle.duration = (uint16_t)led->ms;
 		break;
 	default:
 		return -EINVAL;
@@ -957,9 +952,9 @@ steelseries_write_led(struct ratbag_led *led)
 
 	if (device_version == 1)
 		return steelseries_write_led_v1(led);
-	else if (device_version == 2)
+	if (device_version == 2)
 		return steelseries_write_led_v2(led);
-	else if (device_version == 3)
+	if (device_version == 3)
 		return steelseries_write_led_v3(led);
 
 	return -ENOTSUP;
@@ -983,7 +978,7 @@ steelseries_write_save(struct ratbag_device *device)
 	} else if (device_version == 2) {
 		buf_len = STEELSERIES_REPORT_SIZE;
 		msg.msg.parameters[0] = STEELSERIES_ID_SAVE;
-	} else if (device_version == 3) {
+	} else if (device_version == 3 || device_version == 4) {
 		buf_len = STEELSERIES_REPORT_SIZE;
 		msg.msg.parameters[0] = STEELSERIES_ID_SAVE_PROTOCOL3;
 	} else {
@@ -1003,48 +998,75 @@ steelseries_write_profile(struct ratbag_profile *profile)
 {
 	struct ratbag_resolution *resolution;
 	struct ratbag_button *button;
+	struct ratbag_device *device = profile->device;
 	struct ratbag_led *led;
 	int rc;
 	bool buttons_dirty = false;
 
 	if (profile->rate_dirty) {
+		log_debug(device->ratbag,
+			  "Report rate changed, rewriting\n");
+
 		rc = steelseries_write_report_rate(profile);
-		if (rc != 0)
+		if (rc != 0) {
+			log_error(device->ratbag,
+				  "Failed to write report rate: %s (%d)\n",
+				  strerror(-rc), rc);
 			return rc;
+		}
 	}
 
 	ratbag_profile_for_each_resolution(profile, resolution) {
 		if (!resolution->dirty)
 			continue;
 
+		log_debug(device->ratbag,
+			  "Resolution %d changed, rewriting\n", resolution->index);
+
 		rc = steelseries_write_dpi(resolution);
-		if (rc != 0)
+		if (rc != 0) {
+			log_error(device->ratbag,
+				  "Failed to write resolution: %s (%d)\n",
+				  strerror(-rc), rc);
 			return rc;
+		}
 
 		/* The same hz is used for all resolutions. Only write once. */
-		if (resolution->index > 0)
-			continue;
-
+		break;
 	}
 
 	ratbag_profile_for_each_button(profile, button) {
 		if (button->dirty)
 			buttons_dirty = true;
+
+		log_debug(device->ratbag,
+			  "Button %d changed, rewriting\n", button->index);
 	}
 
 	if (buttons_dirty) {
 		rc = steelseries_write_buttons(profile);
-		if (rc != 0)
+		if (rc != 0) {
+			log_error(device->ratbag,
+				  "Failed to write buttons: %s (%d)\n",
+				  strerror(-rc), rc);
 			return rc;
+		}
 	}
 
 	ratbag_profile_for_each_led(profile, led) {
 		if (!led->dirty)
 			continue;
 
+		log_debug(device->ratbag,
+			  "LED %d changed, rewriting\n", led->index);
+
 		rc = steelseries_write_led(led);
-		if (rc != 0)
+		if (rc != 0) {
+			log_error(device->ratbag,
+				  "Failed to write LED: %s (%d)\n",
+				  strerror(-rc), rc);
 			return rc;
+		}
 	}
 
 	return 0;
@@ -1064,14 +1086,22 @@ steelseries_commit(struct ratbag_device *device)
 			  "Profile %d changed, rewriting\n", profile->index);
 
 		rc = steelseries_write_profile(profile);
-		if (rc)
+		if (rc) {
+			log_error(device->ratbag,
+				  "Failed to write profile: %s (%d)\n",
+				  strerror(-rc), rc);
 			return rc;
+		}
 
 		/* persist the current settings on the device */
 
 		rc = steelseries_write_save(device);
-		if (rc)
+		if (rc) {
+			log_error(device->ratbag,
+				  "Failed to save profile: %s (%d)\n",
+				  strerror(-rc), rc);
 			return rc;
+		}
 	}
 
 	return 0;
@@ -1082,7 +1112,6 @@ steelseries_remove(struct ratbag_device *device)
 {
 	ratbag_close_hidraw_index(device, 0);
 	ratbag_close_hidraw_index(device, 1);
-	free(ratbag_get_drv_data(device));
 }
 
 struct ratbag_driver steelseries_driver = {
