@@ -51,17 +51,17 @@ test_read_button(struct ratbag_button *button)
 	struct ratbag_test_macro_event *e;
 	int idx;
 
-	/* Only take the button types from the first profile */
-	button->type = d->profiles[0].buttons[button->index].button_type;
-
 	switch (b->action_type) {
+	case RATBAG_BUTTON_ACTION_TYPE_NONE:
+		button->action.type = RATBAG_BUTTON_ACTION_TYPE_NONE;
+		break;
 	case RATBAG_BUTTON_ACTION_TYPE_BUTTON:
 		button->action.type = RATBAG_BUTTON_ACTION_TYPE_BUTTON;
 		button->action.action.button = p->buttons[button->index].button;
 		break;
 	case RATBAG_BUTTON_ACTION_TYPE_KEY:
 		button->action.type = RATBAG_BUTTON_ACTION_TYPE_KEY;
-		button->action.action.key.key = p->buttons[button->index].key;
+		button->action.action.key = p->buttons[button->index].key;
 		break;
 	case RATBAG_BUTTON_ACTION_TYPE_MACRO:
 		button->action.type = RATBAG_BUTTON_ACTION_TYPE_MACRO;
@@ -84,6 +84,7 @@ test_read_button(struct ratbag_button *button)
 		button->action.type = RATBAG_BUTTON_ACTION_TYPE_UNKNOWN;
 	}
 
+	ratbag_button_enable_action_type(button, RATBAG_BUTTON_ACTION_TYPE_NONE);
 	ratbag_button_enable_action_type(button, RATBAG_BUTTON_ACTION_TYPE_BUTTON);
 	ratbag_button_enable_action_type(button, RATBAG_BUTTON_ACTION_TYPE_KEY);
 	ratbag_button_enable_action_type(button, RATBAG_BUTTON_ACTION_TYPE_SPECIAL);
@@ -121,11 +122,6 @@ test_read_led(struct ratbag_led *led)
 	led->color.blue = t_led.color.blue;
 	led->ms = t_led.ms;
 	led->brightness = t_led.brightness;
-
-	/* The led type has to be the same anyway so make writing test
-	 * devices easier by always getting the first profile's LED type.
-	 */
-	led->type = d->profiles[0].leds[led->index].type;
 }
 
 static int
@@ -155,8 +151,10 @@ test_read_profile(struct ratbag_profile *profile)
 	p0 = &d->profiles[0];
 	r0 = &p0->resolutions[0];
 
-	for (size_t r = 0; r < ARRAY_LENGTH(p0->report_rates); r++) {
-		if (p0->report_rates[r] > 0)
+	for (size_t report_rate_index = 0;
+	     report_rate_index < ARRAY_LENGTH(p0->report_rates);
+	     report_rate_index++) {
+		if (p0->report_rates[report_rate_index] > 0)
 			nrates++;
 	}
 
@@ -164,7 +162,7 @@ test_read_profile(struct ratbag_profile *profile)
 		ratbag_profile_set_report_rate_list(profile,
 						    p0->report_rates,
 						    nrates);
-	ratbag_profile_set_report_rate(profile, p->hz);
+	profile->hz = p->hz;
 
 	for (i = 0; i < d->num_resolutions; i++) {
 		_cleanup_resolution_ struct ratbag_resolution *res = NULL;
@@ -186,6 +184,7 @@ test_read_profile(struct ratbag_profile *profile)
 		res->is_default = r->dflt;
 		if (r->dflt)
 			default_set = true;
+		res->is_disabled = r->disabled;
 
 		for (size_t j = 0; j < ARRAY_LENGTH(r->caps) && r->caps[j]; j++) {
 			ratbag_resolution_set_cap(res, r->caps[j]);
