@@ -180,7 +180,8 @@ hidpp10_request_command(struct hidpp10_device *dev, union hidpp10_message *msg)
 		command_size = LONG_MESSAGE_LENGTH;
 		break;
 	default:
-		abort();
+		hidpp_log_error(&dev->base, "Incorrect message report id: %02x\n", msg->msg.report_id);
+		return -EINVAL;
 	}
 
 	/* create the expected header */
@@ -1205,7 +1206,7 @@ hidpp10_onboard_profiles_parse_macro(struct hidpp10_device *device,
 	hidpp_log_raw(&device->base, "*** macro starts at (0x%02x, 0x%04x) ***\n", page, offset);
 
 	rc = hidpp10_onboard_profiles_read_macro(device, page, offset, &macro);
-	if (rc) {
+	if (rc < 0) {
 		hidpp_log_raw(&device->base, "hidpp10: failed to read macro\n");
 		return rc;
 	}
@@ -1270,7 +1271,7 @@ hidpp10_onboard_profiles_parse_macro(struct hidpp10_device *device,
 			break;
 		case HIDPP10_MACRO_JUMP_IF_RELEASED_TIMEOUT:
 			m->jump_timeout.timeout = hidpp_be_u16_to_cpu(m->jump_timeout.timeout);
-			hidpp_log_raw(&device->base, "conditional jump to: (0x%02x, 0x%02x) if released whithin %0.03f msecs.\n", m->jump_timeout.page, m->jump_timeout.offset, m->jump_timeout.timeout / 1000.0);
+			hidpp_log_raw(&device->base, "conditional jump to: (0x%02x, 0x%02x) if released within %0.03f msecs.\n", m->jump_timeout.page, m->jump_timeout.offset, m->jump_timeout.timeout / 1000.0);
 			break;
 		case HIDPP10_MACRO_END:
 			break;
@@ -1468,7 +1469,6 @@ hidpp10_read_profile(struct hidpp10_device *dev, uint8_t number)
 	int res;
 	struct hidpp10_profile *profile;
 	union _hidpp10_button_binding *buttons;
-	uint8_t page;
 
 	/* Page 0 is RAM
 	 * Page 1 is the profile directory
@@ -1492,21 +1492,19 @@ hidpp10_read_profile(struct hidpp10_device *dev, uint8_t number)
 	profile = &dev->profiles[number];
 	if (!profile->page) {
 		unsigned long pages = 0xffff;
-		unsigned int i;
 
 		/* pages 0 and 1 are ROM and directory so they are reserved */
 		long_clear_bit(&pages, 0);
 		long_clear_bit(&pages, 1);
 
-		for (i = 0; i < dev->profile_count; i++) {
+		for (size_t i = 0; i < dev->profile_count; i++) {
 			uint8_t page = dev->profiles[i].page;
 
 			assert(page < sizeof(pages) * 8);
 			long_clear_bit(&pages, page);
 		}
 
-		page = ffsl(pages) - 1;
-		profile->page = page;
+		profile->page = ffsl(pages) - 1;
 	}
 
 	switch (dev->profile_type) {
@@ -1525,7 +1523,7 @@ hidpp10_read_profile(struct hidpp10_device *dev, uint8_t number)
 	}
 
 	if (!profile->initialized) {
-		page = profile->page;
+		uint8_t page = profile->page;
 		res = hidpp10_read_page(dev, page, page_data);
 		if (res == -EILSEQ) {
 			/*
@@ -1943,7 +1941,8 @@ hidpp10_set_led_status(struct hidpp10_device *dev,
 			case HIDPP10_LED_STATUS_SLOW_OFF:
 				break;
 			default:
-				abort();
+				hidpp_log_error(&dev->base, "Incorrect LED status: %02x\n", led[i]);
+				return -EINVAL;
 		}
 	}
 
@@ -2703,7 +2702,7 @@ hidpp10_get_extended_pairing_information(struct hidpp10_device *dev,
 }
 
 int
-hidpp10_get_firmare_information(struct hidpp10_device *dev,
+hidpp10_get_firmware_information(struct hidpp10_device *dev,
 				uint8_t *major_out,
 				uint8_t *minor_out,
 				uint8_t *build_out)
