@@ -62,6 +62,7 @@ enum driver {
 	SINOWEALTH,
 	SINOWEALTH_NUBWO,
 	OPENINPUT,
+	MARSGAMING,
 };
 
 struct data_hidpp20 {
@@ -99,7 +100,7 @@ struct data_steelseries {
 struct data_asus {
 	int profile_count;
 	int button_count;
-	int8_t button_mapping[ASUS_MAX_NUM_BUTTON];
+	int button_mapping[ASUS_MAX_NUM_BUTTON];
 	int led_count;
 	int dpi_count;
 	int is_wireless;
@@ -113,6 +114,7 @@ struct ratbag_device_data {
 	char *driver;
 
 	enum driver drivertype;
+	enum ratbag_device_type devicetype;
 
 	union {
 		struct data_hidpp20 hidpp20;
@@ -121,9 +123,6 @@ struct ratbag_device_data {
 		struct data_steelseries steelseries;
 		struct data_asus asus;
 	};
-
-	enum ratbag_led_type led_types[20];
-	size_t nled_types;
 };
 
 static void
@@ -236,6 +235,7 @@ init_data_sinowealth(struct ratbag *ratbag,
 	const char *devices_group = "Driver/sinowealth/devices/";
 
 	GError *error = NULL;
+	int rc;
 
 	size_t group_count = 0;
 	_cleanup_(g_strfreevp) char **groups = g_key_file_get_groups(keyfile, &group_count);
@@ -244,20 +244,32 @@ init_data_sinowealth(struct ratbag *ratbag,
 
 	for (size_t i = 0; i < group_count; ++i) {
 		const char *device_group = groups[i];
-		if (startswith(device_group, devices_group) == NULL)
+		const char* fw_version = startswith(device_group, devices_group);
+		if (fw_version == NULL)
 			continue;
+		rc = (int)strlen(fw_version);
+		if (rc != SINOWEALTH_FW_VERSION_LEN) {
+			log_error(ratbag,
+				  "Group '%s': incorrect firmware version string length `%d` (must be `%d`)\n",
+				  device_group, rc, SINOWEALTH_FW_VERSION_LEN);
+			continue;
+		}
 
 		struct sinowealth_device_data *device = zalloc(sizeof(struct sinowealth_device_data));
 
-		device->button_count = g_key_file_get_integer(keyfile, device_group, "ButtonCount", &error);
+		device->fw_version = strdup_safe(fw_version);
+
+		rc = g_key_file_get_integer(keyfile, device_group, "Buttons", &error);
+		if (rc != 0 && !error) {
+			device->button_count = rc;
+		} else {
+			device->button_count = -1;
+		}
 		g_clear_error(&error);
 
 		device->device_name = g_key_file_get_string(keyfile, device_group, "DeviceName", &error);
 		g_clear_error(&error);
 
-		device->fw_version = g_key_file_get_string(keyfile, device_group, "FwVersion", &error);
-		g_clear_error(&error);
-
 		_cleanup_free_ char *led_type_str = g_key_file_get_string(keyfile, device_group, "LedType", &error);
 		if (led_type_str) {
 			if (streq(led_type_str, "RGB")) {
@@ -274,6 +286,13 @@ init_data_sinowealth(struct ratbag *ratbag,
 		}
 		g_clear_error(&error);
 
+		rc = g_key_file_get_integer(keyfile, device_group, "Profiles", &error);
+		if (rc != 0 && !error) {
+			device->profile_count = rc;
+		} else {
+			device->profile_count = -1;
+		}
+
 		list_insert(&data->sinowealth.supported_devices, &device->link);
 	}
 }
@@ -373,17 +392,17 @@ init_data_asus(struct ratbag *ratbag,
 	_cleanup_(g_strfreevp) char **button_mapping = g_key_file_get_string_list(keyfile, group, "ButtonMapping", &button_mapping_count, &error);
 	if (!error && button_mapping) {
 		for (unsigned int i = 0; i < button_mapping_count; i++) {
-			data->asus.button_mapping[i] = (int8_t) strtoul(button_mapping[i], NULL, 10);
+			data->asus.button_mapping[i] = strtoul(button_mapping[i], NULL, 16);
 		}
 	}
 	g_clear_error(&error);
 
-        int leds = g_key_file_get_integer(keyfile, group, "Leds", &error);
+	int leds = g_key_file_get_integer(keyfile, group, "Leds", &error);
 	if (!error && leds >= 0 && leds <= ASUS_MAX_NUM_LED)
 		data->asus.led_count = leds;
 	g_clear_error(&error);
 
-        int dpis = g_key_file_get_integer(keyfile, group, "Dpis", &error);
+	int dpis = g_key_file_get_integer(keyfile, group, "Dpis", &error);
 	if (!error && dpis >= 2 && dpis <= ASUS_MAX_NUM_DPI)
 		data->asus.dpi_count = dpis;
 	g_clear_error(&error);
@@ -393,12 +412,12 @@ init_data_asus(struct ratbag *ratbag,
 		data->asus.dpi_range = dpi_range_from_string(dpi_range);
 	g_clear_error(&error);
 
-        int wireless = g_key_file_get_integer(keyfile, group, "Wireless", &error);
+	int wireless = g_key_file_get_integer(keyfile, group, "Wireless", &error);
 	if (!error && (wireless == 0 || wireless == 1))
 		data->asus.is_wireless = wireless;
 	g_clear_error(&error);
 
-        gsize quirks_count = 0;
+	gsize quirks_count = 0;
 	_cleanup_(g_strfreevp) char **quirks = g_key_file_get_string_list(keyfile, group, "Quirks", &quirks_count, &error);
 	if (!error && quirks) {
 		for (unsigned int i = 0; i < quirks_count; i++) {
@@ -435,6 +454,7 @@ static const struct driver_map {
 	{ SINOWEALTH, "sinowealth", init_data_sinowealth },
 	{ SINOWEALTH_NUBWO, "sinowealth_nubwo", NULL},
 	{ OPENINPUT, "openinput", NULL },
+	{ MARSGAMING, "marsgaming", NULL },
 };
 
 const char *
@@ -449,13 +469,10 @@ ratbag_device_data_get_name(const struct ratbag_device_data *data)
 	return data->name;
 }
 
-enum ratbag_led_type
-ratbag_device_data_get_led_type(const struct ratbag_device_data *data,
-				unsigned int index)
+enum ratbag_device_type
+ratbag_device_data_get_device_type(const struct ratbag_device_data *data)
 {
-	assert(index < ARRAY_LENGTH(data->led_types));
-
-	return data->led_types[index];
+	return data->devicetype;
 }
 
 struct ratbag_device_data *
@@ -470,11 +487,7 @@ ratbag_device_data_destroy(struct ratbag_device_data *data)
 {
 	switch (data->drivertype) {
 	case HIDPP10:
-		if (data->hidpp10.dpi_list) {
-			free(data->hidpp10.dpi_list->entries);
-			free(data->hidpp10.dpi_list);
-		}
-
+		dpi_list_free(data->hidpp10.dpi_list);
 		free(data->hidpp10.dpi_range);
 		free(data->hidpp10.profile_type);
 		break;
@@ -491,11 +504,7 @@ ratbag_device_data_destroy(struct ratbag_device_data *data)
 		break;
 	}
 	case STEELSERIES:
-		if (data->steelseries.dpi_list) {
-			free(data->steelseries.dpi_list->entries);
-			free(data->steelseries.dpi_list);
-		}
-
+		dpi_list_free(data->steelseries.dpi_list);
 		free(data->steelseries.dpi_range);
 		break;
 	default:
@@ -523,44 +532,6 @@ ratbag_device_data_unref(struct ratbag_device_data *data)
 
 DEFINE_TRIVIAL_CLEANUP_FUNC(struct ratbag_device_data *, ratbag_device_data_unref);
 
-static int
-parse_ledtypes(char **strv, enum ratbag_led_type *types, size_t ntypes)
-{
-	unsigned int i;
-	int count = 0;
-
-	for (i = 0; i < ntypes; i++)
-		types[i] = RATBAG_LED_TYPE_UNKNOWN;
-
-	if (!strv)
-		return count;
-
-	i = 0;
-	while(strv[i]) {
-		const char *s = strv[i];
-
-		if (streq(s, "logo"))
-			types[i] = RATBAG_LED_TYPE_LOGO;
-		else if (streq(s, "side"))
-			types[i] = RATBAG_LED_TYPE_SIDE;
-		else if (streq(s, "dpi"))
-			types[i] = RATBAG_LED_TYPE_SIDE;
-		else if (streq(s, "battery"))
-			types[i] = RATBAG_LED_TYPE_SIDE;
-		else if (streq(s, "wheel"))
-			types[i] = RATBAG_LED_TYPE_WHEEL;
-		else if (streq(s, "switches"))
-			types[i] = RATBAG_LED_TYPE_SWITCHES;
-		else
-			return -1;
-
-		count++;
-		i++;
-	}
-
-	return count;
-}
-
 static bool
 match(const struct input_id *id, char **strv)
 {
@@ -594,8 +565,8 @@ file_data_matches(struct ratbag *ratbag,
 	_cleanup_(g_key_file_freep) GKeyFile *keyfile = NULL;
 	_cleanup_(g_error_freep) GError *error = NULL;
 	_cleanup_(g_strfreevp) char **match_strv = NULL;
-	_cleanup_(g_strfreevp) char **ledtypes_strv = NULL;
 	_cleanup_(ratbag_device_data_unrefp) struct ratbag_device_data *data = NULL;
+	_cleanup_free_ char *devicetype = NULL;
 	int rc;
 
 	keyfile = g_key_file_new();
@@ -645,9 +616,21 @@ file_data_matches(struct ratbag *ratbag,
 		}
 	}
 
-	ledtypes_strv = g_key_file_get_string_list(keyfile, GROUP_DEVICE, "LedTypes", NULL, NULL);
-	if (parse_ledtypes(ledtypes_strv, data->led_types, ARRAY_LENGTH(data->led_types)) < 0) {
-		log_error(ratbag, "Invalid LedTypes string in '%s'\n", basename(path));
+	devicetype = g_key_file_get_string(keyfile, GROUP_DEVICE, "DeviceType", NULL);
+	if (devicetype == NULL) {
+		log_error(ratbag, "No DeviceType found in '%s'\n", basename(path));
+		data->devicetype = TYPE_UNSPECIFIED;
+		return false;
+	}
+	if (streq(devicetype, "keyboard")) {
+		data->devicetype = TYPE_KEYBOARD;
+	} else if (streq(devicetype, "mouse")) {
+		data->devicetype = TYPE_MOUSE;
+	} else if (streq(devicetype, "other")) {
+		data->devicetype = TYPE_OTHER;
+	} else {
+		log_error(ratbag, "Invalid DeviceType '%s' in '%s'\n", devicetype, basename(path));
+		data->devicetype = TYPE_UNSPECIFIED;
 		return false;
 	}
 
@@ -662,7 +645,8 @@ filter_device_files(const struct dirent *entry)
 {
 	const char *SUFFIX = ".device";
 	const char *name = entry->d_name;
-	int len, slen;
+	size_t len = 0;
+	size_t slen = 0;
 
 	if (!name || name[0] == '.')
 		return 0;
@@ -897,7 +881,7 @@ ratbag_device_data_asus_get_button_count(const struct ratbag_device_data *data)
 	return data->asus.button_count;
 }
 
-const int8_t *
+const int *
 ratbag_device_data_asus_get_button_mapping(const struct ratbag_device_data *data)
 {
 	assert(data->drivertype == ASUS);
