@@ -64,7 +64,7 @@ int ratbag_open_hidraw(struct ratbag_device *device);
  *
  * @param device the ratbag device
  * @param enumeration index of the endpoint
- * @param interal index of hidraw array
+ * @param internal index of hidraw array
  *
  * @return 0 on success or a negative errno on error
  */
@@ -94,7 +94,7 @@ void ratbag_close_hidraw(struct ratbag_device *device);
  * Close a hidraw device associated with the device by the internal index.
  *
  * @param device the ratbag device
- * @param interal index of hidraw array
+ * @param internal index of hidraw array
  */
 void ratbag_close_hidraw_index(struct ratbag_device *device, int idx);
 
@@ -108,7 +108,7 @@ void ratbag_close_hidraw_index(struct ratbag_device *device, int idx);
  * @param rtype HID report type
  * @param reqtype HID_REQ_GET_REPORT or HID_REQ_SET_REPORT
  *
- * @return count of data transfered, or a negative errno on error
+ * @return count of data transferred, or a negative errno on error
  *
  * Same behavior as hid_hw_request, but with raw buffers instead.
  */
@@ -127,7 +127,7 @@ int ratbag_hidraw_raw_request(struct ratbag_device *device, unsigned char report
  * @param buf[out] data returned from device
  * @param len length of buf
  *
- * @return count of data transfered, or a negative errno on error
+ * @return count of data transferred, or a negative errno on error
  *
  * Convenience wrapper around ratbag_hidraw_raw_request()
  */
@@ -145,7 +145,7 @@ int ratbag_hidraw_get_feature_report(struct ratbag_device *device, unsigned char
  * @param buf[in] data returned from device
  * @param len length of buf
  *
- * @return count of data transfered, or a negative errno on error
+ * @return count of data transferred, or a negative errno on error
  *
  * Convenience wrapper around ratbag_hidraw_raw_request()
  */
@@ -158,7 +158,7 @@ int ratbag_hidraw_set_feature_report(struct ratbag_device *device, unsigned char
  * @param buf raw data to transfer
  * @param len length of buf
  *
- * @return count of data transfered, or a negative errno on error
+ * @return count of data transferred, or a negative errno on error
  */
 int ratbag_hidraw_output_report(struct ratbag_device *device, uint8_t *buf, size_t len);
 
@@ -171,9 +171,9 @@ int ratbag_hidraw_output_report(struct ratbag_device *device, uint8_t *buf, size
  * @param len length of buf
  * @param filter filter function
  *
- * @return count of data transfered, or a negative errno on error
+ * @return count of data transferred, or a negative errno on error
  */
-int ratbag_hidraw_read_input_report(struct ratbag_device *device, uint8_t *buf, size_t len,
+int ratbag_hidraw_read_input_report(const struct ratbag_device *device, uint8_t *buf, size_t len,
 				 ratbagd_hidraw_filter_t filter);
 
 /**
@@ -186,9 +186,9 @@ int ratbag_hidraw_read_input_report(struct ratbag_device *device, uint8_t *buf,
  * @param hidrawno index of hidraw array
  * @param filter filter function
  *
- * @return count of data transfered, or a negative errno on error
+ * @return count of data transferred, or a negative errno on error
  */
-int ratbag_hidraw_read_input_report_index(struct ratbag_device *device, uint8_t *buf, size_t len, int hidrawno,
+int ratbag_hidraw_read_input_report_index(const struct ratbag_device *device, uint8_t *buf, size_t len, int hidrawno,
 				 ratbagd_hidraw_filter_t filter);
 
 /**
@@ -200,7 +200,7 @@ int ratbag_hidraw_read_input_report_index(struct ratbag_device *device, uint8_t
  * @return 1 if the device has the given report id, 0 otherwise
  */
 int
-ratbag_hidraw_has_report(struct ratbag_device *device, unsigned int report_id);
+ratbag_hidraw_has_report(const struct ratbag_device *device, unsigned int report_id);
 
 /**
  * Gives the usage page of a report with the specified report ID.
@@ -212,7 +212,7 @@ ratbag_hidraw_has_report(struct ratbag_device *device, unsigned int report_id);
  * 0 otherwise
  */
 unsigned int
-ratbag_hidraw_get_usage_page(struct ratbag_device *device, unsigned int report_id);
+ratbag_hidraw_get_usage_page(const struct ratbag_device *device, unsigned int report_id);
 
 /**
  * Gives the usage of a report with the specified report ID.
@@ -224,7 +224,7 @@ ratbag_hidraw_get_usage_page(struct ratbag_device *device, unsigned int report_i
  * 0 otherwise
  */
 unsigned int
-ratbag_hidraw_get_usage(struct ratbag_device *device, unsigned int report_id);
+ratbag_hidraw_get_usage(const struct ratbag_device *device, unsigned int report_id);
 
 /**
  * Tells if a given device has a vendor usage page
@@ -234,7 +234,7 @@ ratbag_hidraw_get_usage(struct ratbag_device *device, unsigned int report_id);
  * @return 1 if the device has a vendor usage page, 0 otherwise
  */
 unsigned int
-ratbag_hidraw_has_vendor_page(struct ratbag_device *device);
+ratbag_hidraw_has_vendor_page(const struct ratbag_device *device);
 
 /**
  * Gives the input key code associated to the keyboard HID usage.
@@ -242,7 +242,7 @@ ratbag_hidraw_has_vendor_page(struct ratbag_device *device);
  * @return the key code of the HID usage or 0.
  */
 unsigned int
-ratbag_hidraw_get_keycode_from_keyboard_usage(struct ratbag_device *device,
+ratbag_hidraw_get_keycode_from_keyboard_usage(const struct ratbag_device *device,
 					      uint8_t hid_code);
 
 /**
@@ -251,7 +251,7 @@ ratbag_hidraw_get_keycode_from_keyboard_usage(struct ratbag_device *device,
  * @return the HID keyboard usage or 0.
  */
 uint8_t
-ratbag_hidraw_get_keyboard_usage_from_keycode(struct ratbag_device *device,
+ratbag_hidraw_get_keyboard_usage_from_keycode(const struct ratbag_device *device,
 					      unsigned keycode);
 
 /**
@@ -260,7 +260,7 @@ ratbag_hidraw_get_keyboard_usage_from_keycode(struct ratbag_device *device,
  * @return the key code of the HID usage or 0.
  */
 unsigned int
-ratbag_hidraw_get_keycode_from_consumer_usage(struct ratbag_device *device,
+ratbag_hidraw_get_keycode_from_consumer_usage(const struct ratbag_device *device,
 					      uint16_t hid_code);
 
 /**
@@ -269,5 +269,5 @@ ratbag_hidraw_get_keycode_from_consumer_usage(struct ratbag_device *device,
  * @return the HID Consumer Control usage or 0.
  */
 uint16_t
-ratbag_hidraw_get_consumer_usage_from_keycode(struct ratbag_device *device,
+ratbag_hidraw_get_consumer_usage_from_keycode(const struct ratbag_device *device,
 					      unsigned keycode);
