@@ -123,7 +123,7 @@ void
 log_buffer(struct ratbag *ratbag,
 	enum ratbag_log_priority priority,
 	const char *header,
-	uint8_t *buf, size_t len)
+	const uint8_t *buf, size_t len)
 {
 	_cleanup_free_ char *output_buf = NULL;
 	char *sep = "";
@@ -196,6 +196,10 @@ ratbag_device_new(struct ratbag *ratbag, struct udev_device *udev_device,
 	device->udev_device = udev_device_ref(udev_device);
 	device->ids = *id;
 	device->data = ratbag_device_data_new_for_id(ratbag, id);
+
+	if (device->data != NULL)
+		device->devicetype = ratbag_device_data_get_device_type(device->data);
+
 	list_init(&device->profiles);
 
 	list_insert(&ratbag->devices, &device->link);
@@ -228,6 +232,7 @@ ratbag_device_destroy(struct ratbag_device *device)
 	ratbag_unref(device->ratbag);
 	ratbag_device_data_unref(device->data);
 	free(device->name);
+	free(device->firmware_version);
 	free(device);
 }
 
@@ -294,6 +299,13 @@ ratbag_sanity_check_device(struct ratbag_device *device)
 					  device->name);
 			goto out;
 		}
+
+		if (profile->dirty) {
+			log_bug_libratbag(ratbag,
+					  "%s: profile is dirty while probing\n",
+					  device->name);
+			/* Don't bail yet, as we may have some drivers that do this. */
+		}
 	}
 
 	/* Require 1 active profile */
@@ -304,29 +316,6 @@ ratbag_sanity_check_device(struct ratbag_device *device)
 		goto out;
 	}
 
-	/* Require LED to be the same type in each profile */
-	ratbag_device_for_each_profile(device, profile) {
-		_cleanup_profile_ struct ratbag_profile *p0 = ratbag_device_get_profile(device, 0);
-		struct ratbag_led *led;
-
-
-		ratbag_profile_for_each_led(profile, led) {
-			_cleanup_led_ struct ratbag_led *p0_led = ratbag_profile_get_led(p0, led->index);
-
-			assert(led->type != RATBAG_LED_TYPE_UNKNOWN);
-
-			if (p0_led->type != led->type) {
-				log_bug_libratbag(ratbag,
-						  "%s: mismatching LED types (profile %d led %d)\n",
-						  device->name,
-						  profile->index,
-						  led->type);
-				goto out;
-			}
-		}
-
-	}
-
 	rc = true;
 
 out:
@@ -380,28 +369,11 @@ error:
 	return false;
 }
 
-static inline bool
-ratbag_driver_fallback_logitech(struct ratbag_device *device,
-				const struct input_id *dev_id)
-{
-	int rc;
-
-	if (dev_id->vendor != USB_VENDOR_ID_LOGITECH)
-		return false;
-
-	rc = ratbag_try_driver(device, dev_id, "hidpp20", NULL);
-	if (!rc)
-		rc = ratbag_try_driver(device, dev_id, "hidpp10", NULL);
-
-	return rc;
-}
-
 bool
 ratbag_assign_driver(struct ratbag_device *device,
 		     const struct input_id *dev_id,
 		     const struct ratbag_test_device *test_device)
 {
-	bool rc;
 	const char *driver_name;
 
 	if (!test_device) {
@@ -411,14 +383,8 @@ ratbag_assign_driver(struct ratbag_device *device,
 		driver_name = "test_driver";
 	}
 
-	if (driver_name) {
-		log_debug(device->ratbag, "device assigned driver %s\n", driver_name);
-		rc = ratbag_try_driver(device, dev_id, driver_name, test_device);
-	} else {
-		rc = ratbag_driver_fallback_logitech(device, dev_id);
-	}
-
-	return rc;
+	log_debug(device->ratbag, "device assigned driver %s\n", driver_name);
+	return ratbag_try_driver(device, dev_id, driver_name, test_device);
 }
 
 static char *
@@ -520,6 +486,12 @@ ratbag_device_get_name(const struct ratbag_device* device)
 	return device->name;
 }
 
+LIBRATBAG_EXPORT enum ratbag_device_type
+ratbag_device_get_device_type(const struct ratbag_device *device)
+{
+	return device->devicetype;
+}
+
 LIBRATBAG_EXPORT const char *
 ratbag_device_get_bustype(const struct ratbag_device *device)
 {
@@ -599,6 +571,7 @@ ratbag_create_context(const struct ratbag_interface *interface,
 	ratbag_register_driver(ratbag, &hidpp10_driver);
 	ratbag_register_driver(ratbag, &logitech_g300_driver);
 	ratbag_register_driver(ratbag, &logitech_g600_driver);
+	ratbag_register_driver(ratbag, &marsgaming_driver);
 	ratbag_register_driver(ratbag, &roccat_driver);
 	ratbag_register_driver(ratbag, &roccat_kone_pure_driver);
 	ratbag_register_driver(ratbag, &roccat_emp_driver);
@@ -644,7 +617,6 @@ ratbag_create_button(struct ratbag_profile *profile, unsigned int index)
 	button->refcount = 0;
 	button->profile = profile;
 	button->index = index;
-	button->type = RATBAG_BUTTON_TYPE_UNKNOWN;
 
 	list_append(&profile->buttons, &button->link);
 
@@ -654,7 +626,6 @@ ratbag_create_button(struct ratbag_profile *profile, unsigned int index)
 static struct ratbag_led *
 ratbag_create_led(struct ratbag_profile *profile, unsigned int index)
 {
-	struct ratbag_device *device = profile->device;
 	struct ratbag_led *led;
 
 	led = zalloc(sizeof(*led));
@@ -662,9 +633,6 @@ ratbag_create_led(struct ratbag_profile *profile, unsigned int index)
 	led->profile = profile;
 	led->index = index;
 	led->colordepth = RATBAG_LED_COLORDEPTH_RGB_888;
-	led->type = RATBAG_LED_TYPE_UNKNOWN;
-	if (device->data)
-		led->type = ratbag_device_data_get_led_type(device->data, led->index);
 
 	list_append(&profile->leds, &led->link);
 
@@ -714,6 +682,8 @@ ratbag_create_profile(struct ratbag_device *device,
 	profile->num_resolutions = 0;
 	profile->is_enabled = true;
 	profile->name = NULL;
+	profile->angle_snapping = -1;
+	profile->debounce = -1;
 
 	list_append(&device->profiles, &profile->link);
 	list_init(&profile->buttons);
@@ -828,6 +798,9 @@ ratbag_profile_set_enabled(struct ratbag_profile *profile, bool enabled)
 	if (!ratbag_profile_has_capability(profile, RATBAG_PROFILE_CAP_DISABLE))
 		return RATBAG_ERROR_CAPABILITY;
 
+	if (profile->is_active && !enabled)
+		return RATBAG_ERROR_VALUE;
+
 	profile->is_enabled = enabled;
 	profile->dirty = true;
 
@@ -835,11 +808,17 @@ ratbag_profile_set_enabled(struct ratbag_profile *profile, bool enabled)
 }
 
 LIBRATBAG_EXPORT bool
-ratbag_profile_is_active(struct ratbag_profile *profile)
+ratbag_profile_is_active(const struct ratbag_profile *profile)
 {
 	return !!profile->is_active;
 }
 
+LIBRATBAG_EXPORT bool
+ratbag_profile_is_dirty(const struct ratbag_profile *profile)
+{
+	return !!profile->dirty;
+}
+
 LIBRATBAG_EXPORT bool
 ratbag_profile_is_enabled(const struct ratbag_profile *profile)
 {
@@ -847,19 +826,19 @@ ratbag_profile_is_enabled(const struct ratbag_profile *profile)
 }
 
 LIBRATBAG_EXPORT unsigned int
-ratbag_device_get_num_profiles(struct ratbag_device *device)
+ratbag_device_get_num_profiles(const struct ratbag_device *device)
 {
 	return device->num_profiles;
 }
 
 LIBRATBAG_EXPORT unsigned int
-ratbag_device_get_num_buttons(struct ratbag_device *device)
+ratbag_device_get_num_buttons(const struct ratbag_device *device)
 {
 	return device->num_buttons;
 }
 
 LIBRATBAG_EXPORT unsigned int
-ratbag_device_get_num_leds(struct ratbag_device *device)
+ratbag_device_get_num_leds(const struct ratbag_device *device)
 {
 	return device->num_leds;
 }
@@ -873,6 +852,12 @@ ratbag_device_commit(struct ratbag_device *device)
 	struct ratbag_resolution *resolution;
 	int rc;
 
+	if (device->driver->commit == NULL) {
+		log_error(device->ratbag,
+			  "Trying to commit with a driver that doesn't support committing\n");
+		return RATBAG_ERROR_CAPABILITY;
+	}
+
 	rc = device->driver->commit(device);
 	if (rc)
 		return RATBAG_ERROR_DEVICE;
@@ -880,6 +865,8 @@ ratbag_device_commit(struct ratbag_device *device)
 	list_for_each(profile, &device->profiles, link) {
 		profile->dirty = false;
 
+		profile->angle_snapping_dirty = false;
+		profile->debounce_dirty = false;
 		profile->rate_dirty = false;
 
 		list_for_each(button, &profile->buttons, link)
@@ -891,6 +878,16 @@ ratbag_device_commit(struct ratbag_device *device)
 		list_for_each(resolution, &profile->resolutions, link)
 			resolution->dirty = false;
 
+		/* TODO: think if this should be moved into `driver-commit`. */
+		if (profile->is_active_dirty && profile->is_active) {
+			if (device->driver->set_active_profile == NULL)
+				return RATBAG_ERROR_IMPLEMENTATION;
+
+			rc = device->driver->set_active_profile(device, profile->index);
+			if (rc)
+				return RATBAG_ERROR_DEVICE;
+		}
+		profile->is_active_dirty = false;
 	}
 
 	return RATBAG_SUCCESS;
@@ -901,25 +898,29 @@ ratbag_profile_set_active(struct ratbag_profile *profile)
 {
 	struct ratbag_device *device = profile->device;
 	struct ratbag_profile *p;
-	int rc;
+
+	if (!profile->is_enabled)
+		return RATBAG_ERROR_VALUE;
 
 	if (device->num_profiles == 1)
 		return RATBAG_SUCCESS;
 
-	assert(device->driver->set_active_profile);
-	rc = device->driver->set_active_profile(device, profile->index);
-	if (rc)
-		return RATBAG_ERROR_DEVICE;
-
-	list_for_each(p, &device->profiles, link)
-		p->is_active = false;
+	list_for_each(p, &device->profiles, link) {
+		if (p->is_active) {
+			p->is_active = false;
+			p->is_active_dirty = true;
+			p->dirty = true;
+		}
+	}
 
 	profile->is_active = true;
+	profile->is_active_dirty = true;
+	profile->dirty = true;
 	return RATBAG_SUCCESS;
 }
 
 LIBRATBAG_EXPORT unsigned int
-ratbag_profile_get_num_resolutions(struct ratbag_profile *profile)
+ratbag_profile_get_num_resolutions(const struct ratbag_profile *profile)
 {
 	return profile->num_resolutions;
 }
@@ -973,21 +974,16 @@ ratbag_resolution_unref(struct ratbag_resolution *resolution)
 }
 
 LIBRATBAG_EXPORT bool
-ratbag_resolution_has_capability(struct ratbag_resolution *resolution,
+ratbag_resolution_has_capability(const struct ratbag_resolution *resolution,
 				 enum ratbag_resolution_capability cap)
 {
-	switch (cap) {
-	case RATBAG_RESOLUTION_CAP_SEPARATE_XY_RESOLUTION:
-		break;
-	default:
-		return 0;
-	}
+	assert(cap <= RATBAG_RESOLUTION_CAP_DISABLE);
 
 	return !!(resolution->capabilities & (1 << cap));
 }
 
 static inline bool
-resolution_has_dpi(struct ratbag_resolution *resolution,
+resolution_has_dpi(const struct ratbag_resolution *resolution,
 		   unsigned int dpi)
 {
 	for (size_t i = 0; i < resolution->ndpis; i++) {
@@ -1056,14 +1052,40 @@ ratbag_profile_set_report_rate(struct ratbag_profile *profile,
 	return RATBAG_SUCCESS;
 }
 
+LIBRATBAG_EXPORT enum ratbag_error_code
+ratbag_profile_set_angle_snapping(struct ratbag_profile *profile,
+				  int value)
+{
+	if (profile->angle_snapping != value) {
+		profile->angle_snapping = value;
+		profile->dirty = true;
+		profile->angle_snapping_dirty = true;
+	}
+
+	return RATBAG_SUCCESS;
+}
+
+LIBRATBAG_EXPORT enum ratbag_error_code
+ratbag_profile_set_debounce(struct ratbag_profile *profile,
+			    int value)
+{
+	if (profile->debounce != value) {
+		profile->debounce = value;
+		profile->dirty = true;
+		profile->debounce_dirty = true;
+	}
+
+	return RATBAG_SUCCESS;
+}
+
 LIBRATBAG_EXPORT int
-ratbag_resolution_get_dpi(struct ratbag_resolution *resolution)
+ratbag_resolution_get_dpi(const struct ratbag_resolution *resolution)
 {
 	return resolution->dpi_x;
 }
 
 LIBRATBAG_EXPORT size_t
-ratbag_resolution_get_dpi_list(struct ratbag_resolution *resolution,
+ratbag_resolution_get_dpi_list(const struct ratbag_resolution *resolution,
 			       unsigned int *resolutions,
 			       size_t nres)
 {
@@ -1081,25 +1103,55 @@ ratbag_resolution_get_dpi_list(struct ratbag_resolution *resolution,
 }
 
 LIBRATBAG_EXPORT int
-ratbag_resolution_get_dpi_x(struct ratbag_resolution *resolution)
+ratbag_resolution_get_dpi_x(const struct ratbag_resolution *resolution)
 {
 	return resolution->dpi_x;
 }
 
 LIBRATBAG_EXPORT int
-ratbag_resolution_get_dpi_y(struct ratbag_resolution *resolution)
+ratbag_resolution_get_dpi_y(const struct ratbag_resolution *resolution)
 {
 	return resolution->dpi_y;
 }
 
 LIBRATBAG_EXPORT int
-ratbag_profile_get_report_rate(struct ratbag_profile *profile)
+ratbag_profile_get_report_rate(const struct ratbag_profile *profile)
 {
 	return profile->hz;
 }
 
+LIBRATBAG_EXPORT int
+ratbag_profile_get_angle_snapping(const struct ratbag_profile *profile)
+{
+	return profile->angle_snapping;
+}
+
+LIBRATBAG_EXPORT int
+ratbag_profile_get_debounce(const struct ratbag_profile *profile)
+{
+	return profile->debounce;
+}
+
 LIBRATBAG_EXPORT size_t
-ratbag_profile_get_report_rate_list(struct ratbag_profile *profile,
+ratbag_profile_get_debounce_list(const struct ratbag_profile *profile,
+				 unsigned int *debounces,
+				 size_t ndebounces)
+{
+	_Static_assert(sizeof(*debounces) == sizeof(*profile->debounces), "type mismatch");
+
+	assert(ndebounces > 0);
+
+	if (profile->ndebounces == 0)
+		return 0;
+
+	memcpy(debounces, profile->debounces,
+	       sizeof(unsigned int) * min(ndebounces, profile->ndebounces));
+
+	return profile->ndebounces;
+}
+
+LIBRATBAG_EXPORT size_t
+ratbag_profile_get_report_rate_list(const struct ratbag_profile *profile,
 				    unsigned int *rates,
 				    size_t nrates)
 {
@@ -1128,6 +1180,11 @@ ratbag_resolution_set_active(struct ratbag_resolution *resolution)
 	struct ratbag_profile *profile = resolution->profile;
 	struct ratbag_resolution *res;
 
+	if (resolution->is_disabled) {
+		log_error(profile->device->ratbag, "%s: setting the active resolution to a disabled resolution is not allowed\n", profile->device->name);
+		return RATBAG_ERROR_VALUE;
+	}
+
 	ratbag_profile_for_each_resolution(profile, res)
 		res->is_active = false;
 
@@ -1137,7 +1194,6 @@ ratbag_resolution_set_active(struct ratbag_resolution *resolution)
 	return RATBAG_SUCCESS;
 }
 
-
 LIBRATBAG_EXPORT bool
 ratbag_resolution_is_default(const struct ratbag_resolution *resolution)
 {
@@ -1150,6 +1206,11 @@ ratbag_resolution_set_default(struct ratbag_resolution *resolution)
 	struct ratbag_profile *profile = resolution->profile;
 	struct ratbag_resolution *other;
 
+	if (resolution->is_disabled) {
+		log_error(profile->device->ratbag, "%s: setting the default resolution to a disabled resolution is not allowed\n", profile->device->name);
+		return RATBAG_ERROR_VALUE;
+	}
+
 	/* Unset the default on the other resolutions */
 	ratbag_profile_for_each_resolution(profile, other) {
 		if (other == resolution || !other->is_default)
@@ -1169,6 +1230,38 @@ ratbag_resolution_set_default(struct ratbag_resolution *resolution)
 	return RATBAG_SUCCESS;
 }
 
+LIBRATBAG_EXPORT bool
+ratbag_resolution_is_disabled(const struct ratbag_resolution *resolution)
+{
+	return !!resolution->is_disabled;
+}
+
+LIBRATBAG_EXPORT enum ratbag_error_code
+ratbag_resolution_set_disabled(struct ratbag_resolution *resolution, bool disable)
+{
+	struct ratbag_profile *profile = resolution->profile;
+
+	if (!ratbag_resolution_has_capability(resolution, RATBAG_RESOLUTION_CAP_DISABLE))
+		return RATBAG_ERROR_CAPABILITY;
+
+	if (disable) {
+		if (resolution->is_active) {
+			log_error(profile->device->ratbag, "%s: disabling the active resolution is not allowed\n", profile->device->name);
+			return RATBAG_ERROR_VALUE;
+		}
+		if (resolution->is_default) {
+			log_error(profile->device->ratbag, "%s: disabling the default resolution is not allowed\n", profile->device->name);
+			return RATBAG_ERROR_VALUE;
+		}
+	}
+
+	resolution->is_disabled = disable;
+	resolution->dirty = true;
+	profile->dirty = true;
+
+	return RATBAG_SUCCESS;
+}
+
 LIBRATBAG_EXPORT struct ratbag_button*
 ratbag_profile_get_button(struct ratbag_profile *profile,
 				   unsigned int index)
@@ -1193,16 +1286,17 @@ ratbag_profile_get_button(struct ratbag_profile *profile,
 }
 
 LIBRATBAG_EXPORT enum ratbag_button_action_type
-ratbag_button_get_action_type(struct ratbag_button *button)
+ratbag_button_get_action_type(const struct ratbag_button *button)
 {
 	return button->action.type;
 }
 
 LIBRATBAG_EXPORT bool
-ratbag_button_has_action_type(struct ratbag_button *button,
+ratbag_button_has_action_type(const struct ratbag_button *button,
 			      enum ratbag_button_action_type action_type)
 {
 	switch (action_type) {
+	case RATBAG_BUTTON_ACTION_TYPE_NONE:
 	case RATBAG_BUTTON_ACTION_TYPE_BUTTON:
 	case RATBAG_BUTTON_ACTION_TYPE_SPECIAL:
 	case RATBAG_BUTTON_ACTION_TYPE_KEY:
@@ -1216,7 +1310,7 @@ ratbag_button_has_action_type(struct ratbag_button *button,
 }
 
 LIBRATBAG_EXPORT unsigned int
-ratbag_button_get_button(struct ratbag_button *button)
+ratbag_button_get_button(const struct ratbag_button *button)
 {
 	if (button->action.type != RATBAG_BUTTON_ACTION_TYPE_BUTTON)
 		return 0;
@@ -1255,7 +1349,7 @@ ratbag_button_set_button(struct ratbag_button *button, unsigned int btn)
 }
 
 LIBRATBAG_EXPORT enum ratbag_button_action_special
-ratbag_button_get_special(struct ratbag_button *button)
+ratbag_button_get_special(const struct ratbag_button *button)
 {
 	if (button->action.type != RATBAG_BUTTON_ACTION_TYPE_SPECIAL)
 		return RATBAG_BUTTON_ACTION_SPECIAL_INVALID;
@@ -1286,23 +1380,16 @@ ratbag_button_set_special(struct ratbag_button *button,
 }
 
 LIBRATBAG_EXPORT unsigned int
-ratbag_button_get_key(struct ratbag_button *button,
-		      unsigned int *modifiers,
-		      size_t *sz)
+ratbag_button_get_key(const struct ratbag_button *button)
 {
 	if (button->action.type != RATBAG_BUTTON_ACTION_TYPE_KEY)
 		return 0;
 
-	/* FIXME: modifiers */
-	*sz = 0;
-	return button->action.action.key.key;
+	return button->action.action.key;
 }
 
 LIBRATBAG_EXPORT enum ratbag_error_code
-ratbag_button_set_key(struct ratbag_button *button,
-		      unsigned int key,
-		      unsigned int *modifiers,
-		      size_t sz)
+ratbag_button_set_key(struct ratbag_button *button, unsigned int key)
 {
 	struct ratbag_button_action action = {0};
 
@@ -1312,10 +1399,8 @@ ratbag_button_set_key(struct ratbag_button *button,
 					   RATBAG_BUTTON_ACTION_TYPE_KEY))
 		return RATBAG_ERROR_CAPABILITY;
 
-	/* FIXME: modifiers */
-
 	action.type = RATBAG_BUTTON_ACTION_TYPE_KEY;
-	action.action.key.key = key;
+	action.action.key = key;
 
 	ratbag_button_set_action(button, &action);
 	button->dirty = true;
@@ -1327,12 +1412,12 @@ ratbag_button_set_key(struct ratbag_button *button,
 LIBRATBAG_EXPORT enum ratbag_error_code
 ratbag_button_disable(struct ratbag_button *button)
 {
-	struct ratbag_button_action action = {0};
-
 	if (!ratbag_button_has_action_type(button,
-					   RATBAG_BUTTON_ACTION_TYPE_KEY))
+					   RATBAG_BUTTON_ACTION_TYPE_NONE))
 		return RATBAG_ERROR_CAPABILITY;
 
+	struct ratbag_button_action action = {0};
+
 	action.type = RATBAG_BUTTON_ACTION_TYPE_NONE;
 
 	ratbag_button_set_action(button, &action);
@@ -1417,13 +1502,13 @@ ratbag_led_unref(struct ratbag_led *led)
 }
 
 LIBRATBAG_EXPORT enum ratbag_led_mode
-ratbag_led_get_mode(struct ratbag_led *led)
+ratbag_led_get_mode(const struct ratbag_led *led)
 {
 	return led->mode;
 }
 
 LIBRATBAG_EXPORT bool
-ratbag_led_has_mode(struct ratbag_led *led,
+ratbag_led_has_mode(const struct ratbag_led *led,
 		    enum ratbag_led_mode mode)
 {
 	assert(mode <= RATBAG_LED_BREATHING);
@@ -1435,19 +1520,19 @@ ratbag_led_has_mode(struct ratbag_led *led,
 }
 
 LIBRATBAG_EXPORT struct ratbag_color
-ratbag_led_get_color(struct ratbag_led *led)
+ratbag_led_get_color(const struct ratbag_led *led)
 {
 	return led->color;
 }
 
 LIBRATBAG_EXPORT int
-ratbag_led_get_effect_duration(struct ratbag_led *led)
+ratbag_led_get_effect_duration(const struct ratbag_led *led)
 {
 	return led->ms;
 }
 
 LIBRATBAG_EXPORT unsigned int
-ratbag_led_get_brightness(struct ratbag_led *led)
+ratbag_led_get_brightness(const struct ratbag_led *led)
 {
 	return led->brightness;
 }
@@ -1471,7 +1556,7 @@ ratbag_led_set_color(struct ratbag_led *led, struct ratbag_color color)
 }
 
 LIBRATBAG_EXPORT enum ratbag_led_colordepth
-ratbag_led_get_colordepth(struct ratbag_led *led)
+ratbag_led_get_colordepth(const struct ratbag_led *led)
 {
 	return led->colordepth;
 }
@@ -1518,7 +1603,7 @@ ratbag_profile_get_led(struct ratbag_profile *profile,
 }
 
 LIBRATBAG_EXPORT const char *
-ratbag_profile_get_name(struct ratbag_profile *profile)
+ratbag_profile_get_name(const struct ratbag_profile *profile)
 {
 	return profile->name;
 }
@@ -1584,6 +1669,19 @@ ratbag_device_get_user_data(const struct ratbag_device *ratbag_device)
 	return ratbag_device->userdata;
 }
 
+LIBRATBAG_EXPORT const char*
+ratbag_device_get_firmware_version(const struct ratbag_device *ratbag_device)
+{
+	return ratbag_device->firmware_version;
+}
+
+LIBRATBAG_EXPORT void
+ratbag_device_set_firmware_version(struct ratbag_device *device, const char* fw)
+{
+	free(device->firmware_version);
+	device->firmware_version = strdup_safe(fw);
+}
+
 LIBRATBAG_EXPORT void*
 ratbag_profile_get_user_data(const struct ratbag_profile *ratbag_profile)
 {
@@ -1685,7 +1783,8 @@ ratbag_button_macro_set_event(struct ratbag_button_macro *m,
 }
 
 LIBRATBAG_EXPORT enum ratbag_macro_event_type
-ratbag_button_macro_get_event_type(struct ratbag_button_macro *macro, unsigned int index)
+ratbag_button_macro_get_event_type(const struct ratbag_button_macro *macro,
+				   unsigned int index)
 {
 	if (index >= MAX_MACRO_EVENTS)
 		return RATBAG_MACRO_EVENT_INVALID;
@@ -1694,9 +1793,10 @@ ratbag_button_macro_get_event_type(struct ratbag_button_macro *macro, unsigned i
 }
 
 LIBRATBAG_EXPORT int
-ratbag_button_macro_get_event_key(struct ratbag_button_macro *m, unsigned int index)
+ratbag_button_macro_get_event_key(const struct ratbag_button_macro *m,
+				  unsigned int index)
 {
-	struct ratbag_macro *macro = &m->macro;
+	const struct ratbag_macro *macro = &m->macro;
 
 	if (index >= MAX_MACRO_EVENTS)
 		return 0;
@@ -1709,10 +1809,10 @@ ratbag_button_macro_get_event_key(struct ratbag_button_macro *m, unsigned int in
 }
 
 LIBRATBAG_EXPORT int
-ratbag_button_macro_get_event_timeout(struct ratbag_button_macro *m,
+ratbag_button_macro_get_event_timeout(const struct ratbag_button_macro *m,
 				      unsigned int index)
 {
-	struct ratbag_macro *macro = &m->macro;
+	const struct ratbag_macro *macro = &m->macro;
 
 	if (index >= MAX_MACRO_EVENTS)
 		return 0;
@@ -1724,13 +1824,13 @@ ratbag_button_macro_get_event_timeout(struct ratbag_button_macro *m,
 }
 
 LIBRATBAG_EXPORT unsigned int
-ratbag_button_macro_get_num_events(struct ratbag_button_macro *macro)
+ratbag_button_macro_get_num_events(const struct ratbag_button_macro *macro)
 {
 	return MAX_MACRO_EVENTS;
 }
 
 LIBRATBAG_EXPORT const char *
-ratbag_button_macro_get_name(struct ratbag_button_macro *macro)
+ratbag_button_macro_get_name(const struct ratbag_button_macro *macro)
 {
 	return macro->macro.name;
 }
@@ -1839,43 +1939,32 @@ ratbag_button_macro_new_from_keycode(struct ratbag_button *button,
 }
 
 int
-ratbag_action_macro_num_keys(struct ratbag_button_action *action)
+ratbag_action_macro_num_keys(const struct ratbag_button_action *action)
 {
-	struct ratbag_macro *macro = action->macro;
-	int i, count;
-
-	count = 0;
-	for (i = 0; i < MAX_MACRO_EVENTS; i++) {
-		struct ratbag_macro_event event;
-
-		event = macro->events[i];
-		if (event.type == RATBAG_MACRO_EVENT_KEY_PRESSED)
-		{
-			switch(event.event.key) {
-			case KEY_LEFTCTRL:
-			case KEY_LEFTSHIFT:
-			case KEY_LEFTALT:
-			case KEY_LEFTMETA:
-			case KEY_RIGHTCTRL:
-			case KEY_RIGHTSHIFT:
-			case KEY_RIGHTALT:
-			case KEY_RIGHTMETA:
-				break;
-			default:
-				count++;
-			}
+	const struct ratbag_macro *macro = action->macro;
+	int count = 0;
+	for (int i = 0; i < MAX_MACRO_EVENTS; i++) {
+		struct ratbag_macro_event event = macro->events[i];
+		if (event.type == RATBAG_MACRO_EVENT_NONE ||
+		    event.type == RATBAG_MACRO_EVENT_INVALID) {
+			break;
+		}
+		if (ratbag_key_is_modifier(event.event.key)) {
+			continue;
+		}
+		if (event.type == RATBAG_MACRO_EVENT_KEY_PRESSED) {
+			count += 1;
 		}
 	}
-
 	return count;
 }
 
 int
-ratbag_action_keycode_from_macro(struct ratbag_button_action *action,
+ratbag_action_keycode_from_macro(const struct ratbag_button_action *action,
 				 unsigned int *key_out,
 				 unsigned int *modifiers_out)
 {
-	struct ratbag_macro *macro = action->macro;
+	const struct ratbag_macro *macro = action->macro;
 	unsigned int key = KEY_RESERVED;
 	unsigned int modifiers = 0;
 	unsigned int i;
